@@ -1,45 +1,29 @@
-/***************************************************************************
+/*
     File                 : JsonFilter.cpp
     Project              : LabPlot
     Description          : JSON I/O-filter.
     --------------------------------------------------------------------
-    --------------------------------------------------------------------
-    Copyright            : (C) 2018 Andrey Cygankov (craftplace.ms@gmail.com)
-    Copyright            : (C) 2018 Alexander Semke (alexander.semke@web.de)
-    Copyright            : (C) 2018-2020 Stefan Gerlach (stefan.gerlach@uni.kn)
+    SPDX-FileCopyrightText: 2018 Andrey Cygankov <craftplace.ms@gmail.com>
+    SPDX-FileCopyrightText: 2018-2020 Alexander Semke <alexander.semke@web.de>
+    SPDX-FileCopyrightText: 2018-2020 Stefan Gerlach <stefan.gerlach@uni.kn>
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *  This program is free software; you can redistribute it and/or modify   *
- *  it under the terms of the GNU General Public License as published by   *
- *  the Free Software Foundation; either version 2 of the License, or      *
- *  (at your option) any later version.                                    *
- *                                                                         *
- *  This program is distributed in the hope that it will be useful,        *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
- *  GNU General Public License for more details.                           *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the Free Software           *
- *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
- *   Boston, MA  02110-1301  USA                                           *
- *                                                                         *
- ***************************************************************************/
 
 #include "backend/datasources/filters/JsonFilter.h"
 #include "backend/datasources/filters/JsonFilterPrivate.h"
 #include "backend/datasources/AbstractDataSource.h"
 #include "backend/core/column/Column.h"
 #include "backend/spreadsheet/Spreadsheet.h"
+#include "backend/lib/XmlStreamReader.h"
+#include "backend/lib/trace.h"
 
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QJsonArray>
 #include <QDataStream>
 #include <QDateTime>
+
 #include <KLocalizedString>
 #include <KFilterDev>
 
@@ -67,17 +51,17 @@ void JsonFilter::readDataFromFile(const QString& fileName, AbstractDataSource* d
 	d->readDataFromFile(fileName, dataSource, importMode);
 }
 
-QVector<QStringList> JsonFilter::preview(const QString& fileName) {
-	return d->preview(fileName);
+QVector<QStringList> JsonFilter::preview(const QString& fileName, int lines) {
+	return d->preview(fileName, lines);
 }
 
-QVector<QStringList> JsonFilter::preview(QIODevice& device) {
-	return d->preview(device);
+QVector<QStringList> JsonFilter::preview(QIODevice& device, int lines) {
+	return d->preview(device, lines);
 }
 
-QVector<QStringList> JsonFilter::preview(const QJsonDocument& doc) {
-	return d->preview(doc);
-}
+// QVector<QStringList> JsonFilter::preview(const QJsonDocument& doc) {
+// 	return d->preview(doc);
+// }
 
 /*!
 writes the content of the data source \c dataSource to the file \c fileName.
@@ -90,15 +74,13 @@ void JsonFilter::write(const QString& fileName, AbstractDataSource* dataSource)
 /*!
 loads the predefined filter settings for \c filterName
 */
-void JsonFilter::loadFilterSettings(const QString& filterName) {
-	Q_UNUSED(filterName);
+void JsonFilter::loadFilterSettings(const QString& /*filterName*/) {
 }
 
 /*!
 saves the current settings as a new filter with the name \c filterName
 */
-void JsonFilter::saveFilterSettings(const QString& filterName) const {
-	Q_UNUSED(filterName);
+void JsonFilter::saveFilterSettings(const QString& /*filterName*/) const {
 }
 
 /*!
@@ -128,6 +110,10 @@ QJsonValue::Type JsonFilter::dataRowType() const {
 	return d->rowType;
 }
 
+void JsonFilter::setModel(QJsonModel* model) {
+	d->model = model;
+}
+
 void JsonFilter::setModelRows(const QVector<int>& rows) {
 	d->modelRows = rows;
 }
@@ -236,12 +222,13 @@ QString JsonFilter::fileInfoString(const QString& fileName) {
 
 	return info;
 }
+
 //#####################################################################
 //################### Private implementation ##########################
 //#####################################################################
-JsonFilterPrivate::JsonFilterPrivate(JsonFilter* owner) : q(owner),
-	model(new QJsonModel()) {}
-//TODO: delete model from memory
+JsonFilterPrivate::JsonFilterPrivate(JsonFilter* owner) : q(owner) {
+
+}
 
 /*!
 returns 1 if row is invalid and 0 otherwise.
@@ -326,7 +313,7 @@ int JsonFilterPrivate::parseColumnModes(const QJsonValue& row, const QString& ro
 
 		switch (columnValue.type()) {
 			case QJsonValue::Double:
-				columnModes << AbstractColumn::ColumnMode::Numeric;
+				columnModes << AbstractColumn::ColumnMode::Double;
 				break;
 			case QJsonValue::String:
 				columnModes << AbstractFileFilter::columnMode(columnValue.toString(), dateTimeFormat, numberFormat);
@@ -345,7 +332,7 @@ int JsonFilterPrivate::parseColumnModes(const QJsonValue& row, const QString& ro
 
 void JsonFilterPrivate::setEmptyValue(int column, int row) {
 	switch (columnModes[column]) {
-		case AbstractColumn::ColumnMode::Numeric:
+		case AbstractColumn::ColumnMode::Double:
 			static_cast<QVector<double>*>(m_dataContainer[column])->operator[](row) = nanValue;
 			break;
 		case AbstractColumn::ColumnMode::Integer:
@@ -369,7 +356,7 @@ void JsonFilterPrivate::setEmptyValue(int column, int row) {
 void JsonFilterPrivate::setValueFromString(int column, int row, const QString& valueString) {
 	QLocale locale(numberFormat);
 	switch (columnModes[column]) {
-		case AbstractColumn::ColumnMode::Numeric: {
+		case AbstractColumn::ColumnMode::Double: {
 			bool isNumber;
 			const double value = locale.toDouble(valueString, &isNumber);
 			static_cast<QVector<double>*>(m_dataContainer[column])->operator[](row) = isNumber ? value : nanValue;
@@ -415,13 +402,11 @@ int JsonFilterPrivate::prepareDeviceToRead(QIODevice& device) {
 		return 1;
 
 	QJsonParseError err;
-	QJsonDocument doc = QJsonDocument::fromJson(device.readAll(), &err);
+	m_doc = QJsonDocument::fromJson(device.readAll(), &err);
 
-	if (err.error != QJsonParseError::NoError || doc.isEmpty())
+	if (err.error != QJsonParseError::NoError || m_doc.isEmpty())
 		return 1;
 
-	if (prepareDocumentToRead(doc) != 0)
-		return 2;
 	// reset to start of file
 	if (!device.isSequential())
 		device.seek(0);
@@ -430,19 +415,32 @@ int JsonFilterPrivate::prepareDeviceToRead(QIODevice& device) {
 }
 
 /*!
-returns 2 if a parse error has occurred and 0 otherwise.
+	determines the relevant part of the full JSON document to be read and its structure.
+	returns \c true if successful, \c false otherwise.
 */
-int JsonFilterPrivate::prepareDocumentToRead(const QJsonDocument& doc) {
-	model->loadJson(doc);
+bool JsonFilterPrivate::prepareDocumentToRead() {
+	PERFTRACE("Prepare the JSON document to read");
 
 	if (modelRows.isEmpty())
-		m_preparedDoc = doc;
+		m_preparedDoc = m_doc;
 	else {
-		QModelIndex index;
-		for (auto& it : modelRows)
-			index = model->index(it, 0, index);
+		if (modelRows.size() == 1)
+			m_preparedDoc = m_doc; //root element selected, use the full document
+		else {
+			//when running tests there is no ImportFileWidget and JsonOptionsWidget available
+			//where the model is created and also passed to JsonFilter. So, we need to create
+			//a model here for in this case.
+			if (!model) {
+				model = new QJsonModel();
+				model->loadJson(m_doc);
+			}
 
-		m_preparedDoc = model->genJsonByIndex(index);
+			QModelIndex index;
+			for (auto& it : modelRows)
+				index = model->index(it, 0, index);
+
+			m_preparedDoc = model->genJsonByIndex(index);
+		}
 	}
 
 	if (!m_preparedDoc.isEmpty()) {
@@ -451,9 +449,9 @@ int JsonFilterPrivate::prepareDocumentToRead(const QJsonDocument& doc) {
 		else if (m_preparedDoc.isObject())
 			containerType = JsonFilter::DataContainerType::Object;
 		else
-			return 2;
+			return false;
 	} else
-		return 2;
+		return false;
 
 	int countRows = 0;
 	int countCols = -1;
@@ -464,15 +462,16 @@ int JsonFilterPrivate::prepareDocumentToRead(const QJsonDocument& doc) {
 	switch (containerType) {
 		case JsonFilter::DataContainerType::Array: {
 			QJsonArray arr = m_preparedDoc.array();
+			int count = arr.count();
 
-			if (arr.count() < startRow)
-				return 2;
+			if (count < startRow)
+				return false;
 
-			int endRowOffset = (endRow == -1 || endRow > arr.count()) ? arr.count() : endRow;
+			int endRowOffset = (endRow == -1 || endRow > count) ? count : endRow;
 			firstRow = *(arr.begin() + (startRow - 1));
 			for (QJsonArray::iterator it = arr.begin() + (startRow - 1); it != arr.begin() + endRowOffset; ++it) {
 				if (checkRow(*it, countCols) != 0)
-					return 2;
+					return false;
 				countRows++;
 			}
 			break;
@@ -481,7 +480,7 @@ int JsonFilterPrivate::prepareDocumentToRead(const QJsonDocument& doc) {
 			QJsonObject obj = m_preparedDoc.object();
 
 			if (obj.count() < startRow)
-				return 2;
+				return false;
 
 			int startRowOffset = startRow - 1;
 			int endRowOffset = (endRow == -1 || endRow > obj.count()) ? obj.count() : endRow;
@@ -489,7 +488,7 @@ int JsonFilterPrivate::prepareDocumentToRead(const QJsonDocument& doc) {
 			firstRowName = (obj.begin() + startRowOffset).key();
 			for (QJsonObject::iterator it = obj.begin() + startRowOffset; it != obj.begin() + endRowOffset; ++it) {
 				if (checkRow(*it, countCols) != 0)
-					return 2;
+					return false;
 				countRows++;
 			}
 			break;
@@ -503,13 +502,13 @@ int JsonFilterPrivate::prepareDocumentToRead(const QJsonDocument& doc) {
 	m_actualCols = endColumn - startColumn + 1 + createIndexEnabled + importObjectNames;
 
 	if (parseColumnModes(firstRow, firstRowName) != 0)
-		return 2;
+		return false;
 
 	DEBUG("start/end column: = " << startColumn << ' ' << endColumn);
 	DEBUG("start/end rows = " << startRow << ' ' << endRow);
 	DEBUG("actual cols/rows = " << m_actualCols << ' ' << m_actualRows);
 
-	return 0;
+	return true;
 }
 
 /*!
@@ -533,37 +532,29 @@ void JsonFilterPrivate::readDataFromDevice(QIODevice& device, AbstractDataSource
 		//TODO: support other modes and vector names
 		m_prepared = true;
 	}
-	importData(dataSource, importMode, lines);
-}
 
-/*!
-reads the content of document \c doc to the data source \c dataSource. Uses the settings defined in the data source.
-*/
-void JsonFilterPrivate::readDataFromDocument(const QJsonDocument& doc, AbstractDataSource* dataSource, AbstractFileFilter::ImportMode importMode, int lines) {
-	if (!m_prepared) {
-		const int docError = prepareDocumentToRead(doc);
-		if (docError != 0) {
-			DEBUG("Document parse error = " << docError);
-			return;
-		}
-		//TODO: support other modes and vector names
-		m_prepared = true;
-	}
-	importData(dataSource, importMode, lines);
+	if (prepareDocumentToRead())
+		importData(dataSource, importMode, lines);
 }
 
 /*!
 import the content of document \c m_preparedDoc to the data source \c dataSource. Uses the settings defined in the data source.
 */
 void JsonFilterPrivate::importData(AbstractDataSource* dataSource, AbstractFileFilter::ImportMode importMode, int lines) {
-	Q_UNUSED(lines)
-
 	m_columnOffset = dataSource->prepareImport(m_dataContainer, importMode, m_actualRows, m_actualCols, vectorNames, columnModes);
 	int rowOffset = startRow - 1;
 	int colOffset = (int)createIndexEnabled + (int)importObjectNames;
 	DEBUG("reading " << m_actualRows << " lines");
 	DEBUG("reading " << m_actualCols << " columns");
 
+	int progressIndex = 0;
+	const float progressInterval = 0.01 * lines; //update on every 1% only
+
+	const auto& array = m_preparedDoc.array();
+	const auto& arrayIterator = array.begin();
+	const auto& object = m_preparedDoc.object();
+	const auto& objectIterator = object.begin();
+
 	for (int i = 0; i < m_actualRows; ++i) {
 		if (createIndexEnabled)
 			static_cast<QVector<int>*>(m_dataContainer[0])->operator[](i) = i + 1;
@@ -571,14 +562,14 @@ void JsonFilterPrivate::importData(AbstractDataSource* dataSource, AbstractFileF
 		QJsonValue row;
 		switch (containerType) {
 		case JsonFilter::DataContainerType::Array:
-			row = *(m_preparedDoc.array().begin() + rowOffset + i);
+			row = *(arrayIterator + rowOffset + i);
 			break;
 		case JsonFilter::DataContainerType::Object:
 			if (importObjectNames) {
-				const QString& rowName = (m_preparedDoc.object().begin() + rowOffset + i).key();
+				const QString& rowName = (objectIterator + rowOffset + i).key();
 				setValueFromString((int)createIndexEnabled, i, rowName);
 			}
-			row = *(m_preparedDoc.object().begin() + rowOffset + i);
+			row = *(objectIterator + rowOffset + i);
 			break;
 		}
 
@@ -602,7 +593,7 @@ void JsonFilterPrivate::importData(AbstractDataSource* dataSource, AbstractFileF
 
 			switch (value.type()) {
 			case QJsonValue::Double:
-				if (columnModes[colOffset + n] == AbstractColumn::ColumnMode::Numeric)
+				if (columnModes[colOffset + n] == AbstractColumn::ColumnMode::Double)
 					static_cast<QVector<double>*>(m_dataContainer[colOffset + n])->operator[](i) = value.toDouble();
 				else
 					setEmptyValue(colOffset + n, i + startRow - 1);
@@ -619,11 +610,19 @@ void JsonFilterPrivate::importData(AbstractDataSource* dataSource, AbstractFileF
 				break;
 			}
 		}
-		emit q->completed(100 * i/m_actualRows);
+
+		//ask to update the progress bar only if we have more than 1000 lines
+		//only in 1% steps
+		progressIndex++;
+		if (m_actualRows > 1000 && progressIndex > progressInterval) {
+			Q_EMIT q->completed(100 * i/m_actualRows);
+			progressIndex = 0;
+			QApplication::processEvents(QEventLoop::AllEvents, 0);
+		}
 	}
 
 	//set the plot designation to 'X' for index and name columns, if available
-	Spreadsheet* spreadsheet = dynamic_cast<Spreadsheet*>(dataSource);
+	auto* spreadsheet = dynamic_cast<Spreadsheet*>(dataSource);
 	if (spreadsheet) {
 		if (createIndexEnabled)
 			spreadsheet->column(m_columnOffset )->setPlotDesignation(AbstractColumn::PlotDesignation::X);
@@ -637,50 +636,55 @@ void JsonFilterPrivate::importData(AbstractDataSource* dataSource, AbstractFileF
 /*!
 generates the preview for the file \c fileName.
 */
-QVector<QStringList> JsonFilterPrivate::preview(const QString& fileName) {
-	KFilterDev device(fileName);
-	return preview(device);
+QVector<QStringList> JsonFilterPrivate::preview(const QString& fileName, int lines) {
+	if (!m_prepared) {
+		KFilterDev device(fileName);
+		return preview(device, lines);
+	} else
+		return preview(lines);
 }
 
 /*!
 generates the preview for device \c device.
 */
-QVector<QStringList> JsonFilterPrivate::preview(QIODevice& device) {
-	const int deviceError = prepareDeviceToRead(device);
-	if (deviceError != 0) {
-		DEBUG("Device error = " << deviceError);
-		return QVector<QStringList>();
+QVector<QStringList> JsonFilterPrivate::preview(QIODevice& device, int lines) {
+	if (!m_prepared) {
+		const int deviceError = prepareDeviceToRead(device);
+		if (deviceError != 0) {
+			DEBUG("Device error = " << deviceError);
+			return {};
+		}
 	}
 
-	return preview();
-}
+	if (prepareDocumentToRead())
+		return preview(lines);
 
-/*!
-generates the preview for document \c doc.
-*/
-QVector<QStringList> JsonFilterPrivate::preview(const QJsonDocument& doc) {
-	if (prepareDocumentToRead(doc) != 0)
-		return QVector<QStringList>();
-	return preview();
+	return {};
 }
 
 /*!
 generates the preview for document \c m_preparedDoc.
 */
-QVector<QStringList> JsonFilterPrivate::preview() {
+QVector<QStringList> JsonFilterPrivate::preview(int lines) {
 	QVector<QStringList> dataStrings;
 	const int rowOffset = startRow - 1;
-	DEBUG("reading " << m_actualRows << " lines");
-	for (int i = 0; i < m_actualRows; ++i) {
+	DEBUG("	Generating preview for " << qMin(lines, m_actualRows)  << " lines");
+
+	const auto& array = m_preparedDoc.array();
+	const auto& arrayIterator = array.begin();
+	const auto& object = m_preparedDoc.object();
+	const auto& objectIterator = object.begin();
+
+	for (int i = 0; i < qMin(lines, m_actualRows); ++i) {
 		QString rowName;
 		QJsonValue row;
 		switch (containerType) {
 			case JsonFilter::DataContainerType::Object:
-				rowName = (m_preparedDoc.object().begin() + rowOffset + i).key();
-				row = *(m_preparedDoc.object().begin() + rowOffset + i);
+				rowName = (objectIterator + rowOffset + i).key();
+				row = *(objectIterator + rowOffset + i);
 				break;
 			case JsonFilter::DataContainerType::Array:
-				row = *(m_preparedDoc.array().begin() + rowOffset + i);
+				row = *(arrayIterator + rowOffset + i);
 				break;
 		}
 
@@ -710,13 +714,9 @@ QVector<QStringList> JsonFilterPrivate::preview() {
 
 			switch (value.type()) {
 			case QJsonValue::Double:
-				if (columnModes[n] == AbstractColumn::ColumnMode::Numeric)
-					lineString += QString::number(value.toDouble(), 'g', 16);
-				else
-					lineString += lineString += QString();
+				lineString += QString::number(value.toDouble(), 'g', 16);
 				break;
 			case QJsonValue::String:
-				//TODO: add parsing string before appending
 				lineString += value.toString();
 				break;
 			case QJsonValue::Array:
@@ -729,7 +729,6 @@ QVector<QStringList> JsonFilterPrivate::preview() {
 			}
 		}
 		dataStrings << lineString;
-		emit q->completed(100 * i/m_actualRows);
 	}
 	return dataStrings;
 }
@@ -737,10 +736,7 @@ QVector<QStringList> JsonFilterPrivate::preview() {
 /*!
 writes the content of \c dataSource to the file \c fileName.
 */
-void JsonFilterPrivate::write(const QString& fileName, AbstractDataSource* dataSource) {
-	Q_UNUSED(fileName);
-	Q_UNUSED(dataSource);
-
+void JsonFilterPrivate::write(const QString& /*fileName*/, AbstractDataSource* /*dataSource*/) {
 	//TODO: saving data to json file not supported yet
 }
 
@@ -777,72 +773,24 @@ void JsonFilter::save(QXmlStreamWriter* writer) const {
 Loads from XML.
 */
 bool JsonFilter::load(XmlStreamReader* reader) {
-	QString attributeWarning = i18n("Attribute '%1' missing or empty, default value is used");
+	KLocalizedString attributeWarning = ki18n("Attribute '%1' missing or empty, default value is used");
 	QXmlStreamAttributes attribs = reader->attributes();
+	QString str;
 
-	QString str = attribs.value("rowType").toString();
-	if (str.isEmpty())
-		reader->raiseWarning(attributeWarning.arg("'rowType'"));
-	else
-		d->rowType = static_cast<QJsonValue::Type>(str.toInt());
-
-	str = attribs.value("dateTimeFormat").toString();
-	if (str.isEmpty())
-		reader->raiseWarning(attributeWarning.arg("'dateTimeFormat'"));
-	else
-		d->dateTimeFormat = str;
-
-	str = attribs.value("numberFormat").toString();
-	if (str.isEmpty())
-		reader->raiseWarning(attributeWarning.arg("'numberFormat'"));
-	else
-		d->numberFormat = static_cast<QLocale::Language >(str.toInt());
-
-	str = attribs.value("createIndex").toString();
-	if (str.isEmpty())
-		reader->raiseWarning(attributeWarning.arg("'createIndex'"));
-	else
-		d->createIndexEnabled = str.toInt();
-
-	str = attribs.value("importObjectNames").toString();
-	if (str.isEmpty())
-		reader->raiseWarning(attributeWarning.arg("'importObjectNames'"));
-	else
-		d->importObjectNames = str.toInt();
-
-	str = attribs.value("nanValue").toString();
-	if (str.isEmpty())
-		reader->raiseWarning(attributeWarning.arg("'nanValue'"));
-	else
-		d->nanValue = str.toDouble();
-
-	str = attribs.value("startRow").toString();
-	if (str.isEmpty())
-		reader->raiseWarning(attributeWarning.arg("'startRow'"));
-	else
-		d->startRow = str.toInt();
-
-	str = attribs.value("endRow").toString();
-	if (str.isEmpty())
-		reader->raiseWarning(attributeWarning.arg("'endRow'"));
-	else
-		d->endRow = str.toInt();
-
-	str = attribs.value("startColumn").toString();
-	if (str.isEmpty())
-		reader->raiseWarning(attributeWarning.arg("'startColumn'"));
-	else
-		d->startColumn = str.toInt();
-
-	str = attribs.value("endColumn").toString();
-	if (str.isEmpty())
-		reader->raiseWarning(attributeWarning.arg("'endColumn'"));
-	else
-		d->endColumn = str.toInt();
+	READ_INT_VALUE("rowType", rowType, QJsonValue::Type);
+	READ_STRING_VALUE("dateTimeFormat", dateTimeFormat);
+	READ_INT_VALUE("numberFormat", numberFormat, QLocale::Language);
+	READ_INT_VALUE("createIndex", createIndexEnabled, bool);
+	READ_INT_VALUE("importObjectNames", importObjectNames, bool);
+	READ_DOUBLE_VALUE("nanValue", nanValue);
+	READ_INT_VALUE("startRow", startRow, int);
+	READ_INT_VALUE("endRow", endRow, int);
+	READ_INT_VALUE("startColumn", startColumn, int);
+	READ_INT_VALUE("endColumn", endColumn, int);
 
 	QStringList list = attribs.value("modelRows").toString().split(';');
 	if (list.isEmpty())
-		reader->raiseWarning(attributeWarning.arg("'modelRows'"));
+		reader->raiseWarning(attributeWarning.subs("'modelRows'").toString());
 	else {
 		d->modelRows = QVector<int>();
 		for (auto& it : list)
