@@ -1,33 +1,15 @@
-/***************************************************************************
-File                 : DatabaseManagerWidget.cpp
-Project              : LabPlot
-Description          : widget for managing database connections
---------------------------------------------------------------------
-Copyright            : (C) 2017-2018 Alexander Semke (alexander.semke@web.de)
-
-***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *  This program is free software; you can redistribute it and/or modify   *
- *  it under the terms of the GNU General Public License as published by   *
- *  the Free Software Foundation; either version 2 of the License, or      *
- *  (at your option) any later version.                                    *
- *                                                                         *
- *  This program is distributed in the hope that it will be useful,        *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
- *  GNU General Public License for more details.                           *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the Free Software           *
- *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
- *   Boston, MA  02110-1301  USA                                           *
- *                                                                         *
- ***************************************************************************/
+/*
+    File                 : DatabaseManagerWidget.cpp
+    Project              : LabPlot
+    Description          : widget for managing database connections
+    --------------------------------------------------------------------
+    SPDX-FileCopyrightText: 2017-2018 Alexander Semke <alexander.semke@web.de>
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "DatabaseManagerWidget.h"
 #include "backend/lib/macros.h"
+#include "kdefrontend/GuiTools.h"
 
 #include <KConfig>
 #include <KConfigGroup>
@@ -41,9 +23,9 @@ Copyright            : (C) 2017-2018 Alexander Semke (alexander.semke@web.de)
 #include <QSqlError>
 
 #ifdef HAVE_KF5_SYNTAX_HIGHLIGHTING
-#include <KF5/KSyntaxHighlighting/SyntaxHighlighter>
-#include <KF5/KSyntaxHighlighting/Definition>
-#include <KF5/KSyntaxHighlighting/Theme>
+#include <KSyntaxHighlighting/SyntaxHighlighter>
+#include <KSyntaxHighlighting/Definition>
+#include <KSyntaxHighlighting/Theme>
 #endif
 
 /*!
@@ -84,7 +66,7 @@ DatabaseManagerWidget::DatabaseManagerWidget(QWidget* parent, QString conn) : QW
 	connect(ui.leDatabase, &QLineEdit::textChanged, this, &DatabaseManagerWidget::databaseNameChanged);
 	connect(ui.leHost, &QLineEdit::textChanged, this, &DatabaseManagerWidget::hostChanged);
 	connect(ui.sbPort, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &DatabaseManagerWidget::portChanged);
-	connect(ui.chkCustomConnection, &QCheckBox::stateChanged, this, &DatabaseManagerWidget::customConnectionEnabledChanged);
+	connect(ui.chkCustomConnection, &QCheckBox::toggled, this, &DatabaseManagerWidget::customConnectionEnabledChanged);
 	connect(ui.teCustomConnection, &QPlainTextEdit::textChanged, this, &DatabaseManagerWidget::customConnectionChanged);
 	connect(ui.leUserName, &QLineEdit::textChanged, this, &DatabaseManagerWidget::userNameChanged);
 	connect(ui.lePassword, &QLineEdit::textChanged, this, &DatabaseManagerWidget::passwordChanged);
@@ -95,8 +77,7 @@ DatabaseManagerWidget::DatabaseManagerWidget(QWidget* parent, QString conn) : QW
 QString DatabaseManagerWidget::connection() const {
 	if (ui.lwConnections->currentItem())
 		return ui.lwConnections->currentItem()->text();
-	else
-		return QString();
+	return {};
 }
 
 /*!
@@ -152,17 +133,17 @@ void DatabaseManagerWidget::nameChanged(const QString& name) {
 	}
 
 	if (unique) {
-		ui.leName->setStyleSheet(QString());
+		GuiTools::highlight(ui.leName, false);
 		if (auto item = ui.lwConnections->currentItem()) {
 			item->setText(name);
 
 			if (!m_initializing) {
 				m_current_connection->name = name;
-				emit changed();
+				Q_EMIT changed();
 			}
 		}
 	} else
-		ui.leName->setStyleSheet("QLineEdit{background: red;}");
+		GuiTools::highlight(ui.leName, true);
 }
 
 void DatabaseManagerWidget::driverChanged() {
@@ -225,13 +206,13 @@ void DatabaseManagerWidget::driverChanged() {
 
 	if (m_current_connection)
 		m_current_connection->driver = driver;
-	emit changed();
+	Q_EMIT changed();
 }
 
 void DatabaseManagerWidget::selectFile() {
 	KConfigGroup conf(KSharedConfig::openConfig(), QLatin1String("DatabaseManagerWidget"));
 	QString dir = conf.readEntry(QLatin1String("LastDir"), "");
-	QString path = QFileDialog::getOpenFileName(this, i18n("Select the database file"), dir);
+	QString path = QFileDialog::getOpenFileName(this, i18nc("@title:window", "Select the Database File"), dir);
 	if (path.isEmpty())
 		return; //cancel was clicked in the file-dialog
 
@@ -255,7 +236,7 @@ void DatabaseManagerWidget::hostChanged() {
 	//don't allow to try to connect if no hostname provided
 	ui.bTestConnection->setEnabled( !ui.leHost->text().simplified().isEmpty() );
 
-	emit changed();
+	Q_EMIT changed();
 }
 
 void DatabaseManagerWidget::portChanged() {
@@ -264,7 +245,7 @@ void DatabaseManagerWidget::portChanged() {
 
 	if (m_current_connection)
 		m_current_connection->port = ui.sbPort->value();
-	emit changed();
+	Q_EMIT changed();
 }
 
 void DatabaseManagerWidget::databaseNameChanged() {
@@ -279,10 +260,7 @@ void DatabaseManagerWidget::databaseNameChanged() {
 
 		if (!dbName.isEmpty()) {
 			bool fileExists = QFile::exists(dbName);
-			if (fileExists)
-				ui.leDatabase->setStyleSheet(QString());
-			else
-				ui.leDatabase->setStyleSheet(QStringLiteral("QLineEdit{background:red;}"));
+			GuiTools::highlight(ui.leName, !fileExists);
 		} else {
 			ui.leDatabase->setStyleSheet(QString());
 		}
@@ -298,30 +276,30 @@ void DatabaseManagerWidget::databaseNameChanged() {
 
 	if (m_current_connection)
 		m_current_connection->dbName = dbName;
-	emit changed();
+	Q_EMIT changed();
 }
 
-void DatabaseManagerWidget::customConnectionEnabledChanged(int state) {
+void DatabaseManagerWidget::customConnectionEnabledChanged(bool state) {
 	//in case custom connection string is provided:
 	//disable the line edit for the database name
 	//and hide the textedit for the connection string
-	ui.leDatabase->setEnabled(state != Qt::Checked);
-	ui.teCustomConnection->setVisible(state == Qt::Checked);
+	ui.leDatabase->setEnabled(!state);
+	ui.teCustomConnection->setVisible(state);
 
-	if (state == Qt::Checked)
+	if (state)
 		ui.teCustomConnection->setFocus();
 	else
 		ui.leDatabase->setFocus();
 
 	if (m_current_connection)
-		m_current_connection->customConnectionEnabled = (state == Qt::Checked);
-	emit changed();
+		m_current_connection->customConnectionEnabled = state;
+	Q_EMIT changed();
 }
 
 void DatabaseManagerWidget::customConnectionChanged() {
 	if (m_current_connection)
 		m_current_connection->customConnectionString = ui.teCustomConnection->toPlainText();
-	emit changed();
+	Q_EMIT changed();
 }
 
 void DatabaseManagerWidget::userNameChanged() {
@@ -330,7 +308,7 @@ void DatabaseManagerWidget::userNameChanged() {
 
 	if (m_current_connection)
 		m_current_connection->userName = ui.leUserName->text();
-	emit changed();
+	Q_EMIT changed();
 }
 
 void DatabaseManagerWidget::passwordChanged() {
@@ -339,7 +317,7 @@ void DatabaseManagerWidget::passwordChanged() {
 
 	if (m_current_connection)
 		m_current_connection->password = ui.lePassword->text();
-	emit changed();
+	Q_EMIT changed();
 }
 
 void DatabaseManagerWidget::addConnection() {
@@ -416,7 +394,7 @@ void DatabaseManagerWidget::deleteConnection() {
 		m_initializing = false;
 	}
 
-	emit changed();
+	Q_EMIT changed();
 }
 
 void DatabaseManagerWidget::loadConnections() {
