@@ -1,36 +1,17 @@
-/***************************************************************************
+/*
     File                 : ImportDialog.cc
     Project              : LabPlot
     Description          : import file data dialog
     --------------------------------------------------------------------
-    Copyright            : (C) 2008-2019 Alexander Semke (alexander.semke@web.de)
-    Copyright            : (C) 2008-2015 by Stefan Gerlach (stefan.gerlach@uni.kn)
+    SPDX-FileCopyrightText: 2008-2019 Alexander Semke <alexander.semke@web.de>
+    SPDX-FileCopyrightText: 2008-2015 Stefan Gerlach <stefan.gerlach@uni.kn>
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *  This program is free software; you can redistribute it and/or modify   *
- *  it under the terms of the GNU General Public License as published by   *
- *  the Free Software Foundation; either version 2 of the License, or      *
- *  (at your option) any later version.                                    *
- *                                                                         *
- *  This program is distributed in the hope that it will be useful,        *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
- *  GNU General Public License for more details.                           *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the Free Software           *
- *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
- *   Boston, MA  02110-1301  USA                                           *
- *                                                                         *
- ***************************************************************************/
 
 #include "ImportFileDialog.h"
 #include "ImportFileWidget.h"
 #include "backend/core/AspectTreeModel.h"
-#include "backend/datasources/LiveDataSource.h"
 #include "backend/datasources/filters/AbstractFileFilter.h"
 #include "backend/datasources/filters/filters.h"
 #include "backend/spreadsheet/Spreadsheet.h"
@@ -43,10 +24,11 @@
 #include "backend/datasources/MQTTClient.h"
 #endif
 
+#include <KLocalizedString>
 #include <KMessageBox>
+#include <KMessageWidget>
 #include <KSharedConfig>
 #include <KWindowConfig>
-#include <KLocalizedString>
 
 #include <QDialogButtonBox>
 #include <QElapsedTimer>
@@ -73,7 +55,7 @@ ImportFileDialog::ImportFileDialog(MainWin* parent, bool liveDataSource, const Q
 	vLayout->addWidget(m_importFileWidget);
 
 	//dialog buttons
-	QDialogButtonBox* buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Reset |QDialogButtonBox::Cancel);
+	auto* buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Reset |QDialogButtonBox::Cancel);
 	okButton = buttonBox->button(QDialogButtonBox::Ok);
 	m_optionsButton = buttonBox->button(QDialogButtonBox::Reset); //we highjack the default "Reset" button and use if for showing/hiding the options
 	okButton->setEnabled(false); //ok is only available if a valid container was selected
@@ -116,9 +98,7 @@ ImportFileDialog::ImportFileDialog(MainWin* parent, bool liveDataSource, const Q
 	connect(m_importFileWidget, static_cast<void (ImportFileWidget::*)()>(&ImportFileWidget::sourceTypeChanged), this, &ImportFileDialog::checkOkButton);
 	connect(m_importFileWidget, &ImportFileWidget::hostChanged, this, &ImportFileDialog::checkOkButton);
 	connect(m_importFileWidget, &ImportFileWidget::portChanged, this, &ImportFileDialog::checkOkButton);
-	//TODO: do we really need to check the ok button when the preview was refreshed?
-	//If not, remove this together with the previewRefreshed signal in ImportFileWidget
-	//connect(m_importFileWidget, &ImportFileWidget::previewRefreshed, this, &ImportFileDialog::checkOkButton);
+	connect(m_importFileWidget, &ImportFileWidget::error, this, &ImportFileDialog::showErrorMessage);
 #ifdef HAVE_MQTT
 	connect(m_importFileWidget, &ImportFileWidget::subscriptionsChanged, this, &ImportFileDialog::checkOkButton);
 	connect(m_importFileWidget, &ImportFileWidget::checkFileType, this, &ImportFileDialog::checkOkButton);
@@ -140,15 +120,15 @@ ImportFileDialog::~ImportFileDialog() {
 	KWindowConfig::saveWindowSize(windowHandle(), conf);
 }
 
-int ImportFileDialog::sourceType() const {
-	return static_cast<int>(m_importFileWidget->currentSourceType());
+LiveDataSource::SourceType ImportFileDialog::sourceType() const {
+	return m_importFileWidget->currentSourceType();
 }
 
 /*!
   triggers data import to the live data source \c source
 */
 void ImportFileDialog::importToLiveDataSource(LiveDataSource* source, QStatusBar* statusBar) const {
-	DEBUG("ImportFileDialog::importToLiveDataSource()");
+	DEBUG(Q_FUNC_INFO);
 	m_importFileWidget->saveSettings(source);
 
 	//show a progress bar in the status bar
@@ -185,7 +165,7 @@ void ImportFileDialog::importToMQTT(MQTTClient* client) const{
   triggers data import to the currently selected data container
 */
 void ImportFileDialog::importTo(QStatusBar* statusBar) const {
-	DEBUG("ImportFileDialog::importTo()");
+	DEBUG(Q_FUNC_INFO);
 	QDEBUG("	cbAddTo->currentModelIndex() =" << cbAddTo->currentModelIndex());
 	AbstractAspect* aspect = static_cast<AbstractAspect*>(cbAddTo->currentModelIndex().internalPointer());
 	if (!aspect) {
@@ -219,33 +199,35 @@ void ImportFileDialog::importTo(QStatusBar* statusBar) const {
 	timer.start();
 
 	if (aspect->inherits(AspectType::Matrix)) {
-		DEBUG("ImportFileDialog::importTo(): to Matrix");
+		DEBUG(Q_FUNC_INFO <<", to Matrix");
 		auto* matrix = qobject_cast<Matrix*>(aspect);
 		filter->readDataFromFile(fileName, matrix, mode);
 	} else if (aspect->inherits(AspectType::Spreadsheet)) {
-		DEBUG("ImportFileDialog::importTo(): to Spreadsheet");
+		DEBUG(Q_FUNC_INFO << ", to Spreadsheet");
 		auto* spreadsheet = qobject_cast<Spreadsheet*>(aspect);
-		DEBUG(" Calling filter->readDataFromFile() with spreadsheet " << spreadsheet);
 		filter->readDataFromFile(fileName, spreadsheet, mode);
 	} else if (aspect->inherits(AspectType::Workbook)) {
-		DEBUG("ImportFileDialog::importTo(): to Workbook");
+		DEBUG(Q_FUNC_INFO << ", to Workbook");
 		auto* workbook = static_cast<Workbook*>(aspect);
 		workbook->setUndoAware(false);
 		auto sheets = workbook->children<AbstractAspect>();
 
 		AbstractFileFilter::FileType fileType = m_importFileWidget->currentFileType();
-		// multiple data sets/variables for HDF5, NetCDF and ROOT
+		// multiple data sets/variables for special types
 		if (fileType == AbstractFileFilter::FileType::HDF5 || fileType == AbstractFileFilter::FileType::NETCDF ||
-			fileType == AbstractFileFilter::FileType::ROOT) {
+			fileType == AbstractFileFilter::FileType::ROOT || fileType == AbstractFileFilter::FileType::MATIO) {
 			QStringList names;
 			if (fileType == AbstractFileFilter::FileType::HDF5)
 				names = m_importFileWidget->selectedHDF5Names();
 			else if (fileType == AbstractFileFilter::FileType::NETCDF)
 				names = m_importFileWidget->selectedNetCDFNames();
+			else if (fileType == AbstractFileFilter::FileType::MATIO)
+				names = m_importFileWidget->selectedMatioNames();
 			else
 				names = m_importFileWidget->selectedROOTNames();
 
 			int nrNames = names.size(), offset = sheets.size();
+			//QDEBUG(Q_FUNC_INFO << ", selected names: " << names)
 
 			//TODO: think about importing multiple sets into one sheet
 
@@ -260,7 +242,7 @@ void ImportFileDialog::importTo(QStatusBar* statusBar) const {
 				// delete the unneeded spreadsheets
 				if (offset > nrNames) {
 					for (int i = nrNames; i < offset; i++)
-						sheets[i]->remove();
+						sheets.at(i)->remove();
 					offset = nrNames;
 				}
 
@@ -269,7 +251,7 @@ void ImportFileDialog::importTo(QStatusBar* statusBar) const {
 					//HDF5 variable names contain the whole path, remove it and keep the name only
 					QString sheetName = names.at(i);
 					if (fileType == AbstractFileFilter::FileType::HDF5)
-						sheetName = names[i].mid(names[i].lastIndexOf("/") + 1);
+						sheetName = names.at(i).mid(names.at(i).lastIndexOf("/") + 1);
 
 					auto* sheet = sheets.at(i);
 					sheet->setUndoAware(false);
@@ -283,11 +265,11 @@ void ImportFileDialog::importTo(QStatusBar* statusBar) const {
 				//HDF5 variable names contain the whole path, remove it and keep the name only
 				QString sheetName = names.at(i);
 				if (fileType == AbstractFileFilter::FileType::HDF5)
-					sheetName = names[i].mid(names[i].lastIndexOf("/") + 1);
+					sheetName = names.at(i).mid(names.at(i).lastIndexOf("/") + 1);
 
 				auto* spreadsheet = new Spreadsheet(sheetName);
 				if (mode == AbstractFileFilter::ImportMode::Prepend && !sheets.isEmpty())
-					workbook->insertChildBefore(spreadsheet, sheets[0]);
+					workbook->insertChildBefore(spreadsheet, sheets.at(0));
 				else
 					workbook->addChildFast(spreadsheet);
 			}
@@ -296,18 +278,20 @@ void ImportFileDialog::importTo(QStatusBar* statusBar) const {
 			if (mode != AbstractFileFilter::ImportMode::Append)
 				offset = 0;
 
-			// import all sets to a different sheet
+			// import every set to a different sheet
 			sheets = workbook->children<AbstractAspect>();
 			for (int i = 0; i < nrNames; ++i) {
 				if (fileType == AbstractFileFilter::FileType::HDF5)
-					static_cast<HDF5Filter*>(filter)->setCurrentDataSetName(names[i]);
+					static_cast<HDF5Filter*>(filter)->setCurrentDataSetName(names.at(i));
 				else if (fileType == AbstractFileFilter::FileType::NETCDF)
-					static_cast<NetCDFFilter*>(filter)->setCurrentVarName(names[i]);
-				else
-					static_cast<ROOTFilter*>(filter)->setCurrentObject(names[i]);
+					static_cast<NetCDFFilter*>(filter)->setCurrentVarName(names.at(i));
+				else if (fileType == AbstractFileFilter::FileType::MATIO)
+					static_cast<MatioFilter*>(filter)->setCurrentVarName(names.at(i));
+				else if (fileType == AbstractFileFilter::FileType::ROOT)
+					static_cast<ROOTFilter*>(filter)->setCurrentObject(names.at(i));
 
 				int index = i + offset;
-				filter->readDataFromFile(fileName, qobject_cast<Spreadsheet*>(sheets[index]));
+				filter->readDataFromFile(fileName, qobject_cast<Spreadsheet*>(sheets.at(index)));
 			}
 
 			workbook->setUndoAware(true);
@@ -361,7 +345,7 @@ void ImportFileDialog::checkOnFitsTableToMatrix(const bool enable) {
 }
 
 void ImportFileDialog::checkOkButton() {
-	DEBUG("ImportFileDialog::checkOkButton()");
+	DEBUG(Q_FUNC_INFO);
 	if (cbAddTo) { //only check for the target container when no file data source is being added
 		QDEBUG(" cbAddTo->currentModelIndex() = " << cbAddTo->currentModelIndex());
 		AbstractAspect* aspect = static_cast<AbstractAspect*>(cbAddTo->currentModelIndex().internalPointer());
@@ -383,24 +367,27 @@ void ImportFileDialog::checkOkButton() {
 		}
 	}
 
-	QString fileName = m_importFileWidget->fileName();
-#ifdef HAVE_WINDOWS
-	if (!fileName.isEmpty() && fileName.at(1) != QLatin1String(":"))
-#else
-	if (!fileName.isEmpty() && fileName.at(0) != QLatin1String("/"))
-#endif
-		fileName = QDir::homePath() + QLatin1String("/") + fileName;
+	QString fileName = ImportFileWidget::absolutePath(m_importFileWidget->fileName());
+	if (fileName.isEmpty())
+		return;
 
 	DEBUG("Data Source Type: " << ENUM_TO_STRING(LiveDataSource, SourceType, m_importFileWidget->currentSourceType()));
 	switch (m_importFileWidget->currentSourceType()) {
 	case LiveDataSource::SourceType::FileOrPipe: {
-		DEBUG("	fileName = " << fileName.toUtf8().constData());
+		DEBUG("	fileName = " << qPrintable(fileName));
 		const bool enable = QFile::exists(fileName);
 		okButton->setEnabled(enable);
-		if (enable)
+		if (enable) {
 			okButton->setToolTip(i18n("Close the dialog and import the data."));
-		else
-			okButton->setToolTip(i18n("Provide an existing file."));
+			showErrorMessage(QString());
+		} else {
+			QString msg = i18n("The provided file doesn't exist.");
+			okButton->setToolTip(msg);
+
+			//suppress the error widget when the dialog is opened the first time.
+			//show only the error widget if the file was really a non-existing file.
+			showErrorMessage(msg);
+		}
 
 		break;
 	}
@@ -420,14 +407,18 @@ void ImportFileDialog::checkOkButton() {
 				// read-only socket is disconnected immediately (no waitForDisconnected())
 				okButton->setEnabled(true);
 				okButton->setToolTip(i18n("Close the dialog and import the data."));
+				showErrorMessage(QString());
 			} else {
-				DEBUG("failed connect to local socket - " << STDSTRING(lsocket.errorString()));
 				okButton->setEnabled(false);
-				okButton->setToolTip(i18n("Could not connect to the provided local socket."));
+				QString msg = i18n("Could not connect to the provided local socket. Error: %1.", lsocket.errorString());
+				okButton->setToolTip(msg);
+				showErrorMessage(msg);
 			}
 		} else {
 			okButton->setEnabled(false);
-			okButton->setToolTip(i18n("Selected local socket does not exist."));
+			QString msg = i18n("Could not connect to the provided local socket. The socket does not exist.");
+			okButton->setToolTip(msg);
+			showErrorMessage(msg);
 		}
 
 		break;
@@ -440,15 +431,19 @@ void ImportFileDialog::checkOkButton() {
 			if (socket.waitForConnected()) {
 				okButton->setEnabled(true);
 				okButton->setToolTip(i18n("Close the dialog and import the data."));
+				showErrorMessage(QString());
 				socket.disconnectFromHost();
 			} else {
-				DEBUG("failed to connect to TCP socket - " << STDSTRING(socket.errorString()));
 				okButton->setEnabled(false);
-				okButton->setToolTip(i18n("Could not connect to the provided TCP socket."));
+				QString msg = i18n("Could not connect to the provided TCP socket. Error: %1.", socket.errorString());
+				okButton->setToolTip(msg);
+				showErrorMessage(msg);
 			}
 		} else {
 			okButton->setEnabled(false);
-			okButton->setToolTip(i18n("Either the host name or the port number is missing."));
+			QString msg = i18n("Either the host name or the port number is missing.");
+			okButton->setToolTip(msg);
+			showErrorMessage(msg);
 		}
 		break;
 	}
@@ -461,12 +456,14 @@ void ImportFileDialog::checkOkButton() {
 			if (socket.waitForConnected()) {
 				okButton->setEnabled(true);
 				okButton->setToolTip(i18n("Close the dialog and import the data."));
+				showErrorMessage(QString());
 				socket.disconnectFromHost();
 				// read-only socket is disconnected immediately (no waitForDisconnected())
 			} else {
-				DEBUG("failed to connect to UDP socket - " << STDSTRING(socket.errorString()));
 				okButton->setEnabled(false);
-				okButton->setToolTip(i18n("Could not connect to the provided UDP socket."));
+				QString msg = i18n("Could not connect to the provided UDP socket. Error: %1.", socket.errorString());
+				okButton->setToolTip(msg);
+				showErrorMessage(msg);
 			}
 		} else {
 			okButton->setEnabled(false);
@@ -492,14 +489,18 @@ void ImportFileDialog::checkOkButton() {
 			okButton->setEnabled(serialPortOpened);
 			if (serialPortOpened) {
 				okButton->setToolTip(i18n("Close the dialog and import the data."));
+				showErrorMessage(QString());
 				serialPort.close();
 			} else {
-				DEBUG("Could not connect to the provided serial port");
-				okButton->setToolTip(i18n("Could not connect to the provided serial port."));
+				QString msg = i18n("Could not connect to the provided serial port.");
+				okButton->setToolTip(msg);
+				showErrorMessage(msg);
 			}
 		} else {
 			okButton->setEnabled(false);
-			okButton->setToolTip(i18n("Serial port number is missing."));
+			QString msg = i18n("Serial port number is missing.");
+			okButton->setToolTip(msg);
+			showErrorMessage(msg);
 		}
 #endif
 		break;
@@ -507,6 +508,7 @@ void ImportFileDialog::checkOkButton() {
 	case LiveDataSource::SourceType::MQTT: {
 #ifdef HAVE_MQTT
 		const bool enable = m_importFileWidget->isMqttValid();
+		showErrorMessage(QString());
 		if (enable) {
 			okButton->setEnabled(true);
 			okButton->setToolTip(i18n("Close the dialog and import the data."));
@@ -524,3 +526,18 @@ void ImportFileDialog::checkOkButton() {
 QString ImportFileDialog::selectedObject() const {
 	return m_importFileWidget->selectedObject();
 }
+
+void ImportFileDialog::showErrorMessage(const QString& message) {
+	if (message.isEmpty()) {
+		if (m_messageWidget && m_messageWidget->isVisible())
+			m_messageWidget->close();
+	} else {
+		if (!m_messageWidget) {
+			m_messageWidget = new KMessageWidget(this);
+			m_messageWidget->setMessageType(KMessageWidget::Error);
+			vLayout->insertWidget(vLayout->count() - 1, m_messageWidget);
+		}
+		m_messageWidget->setText(message);
+        m_messageWidget->animatedShow();
+	}
+}
