@@ -1,36 +1,17 @@
-/***************************************************************************
+/*
     File                 : ImportSQLDatabaseWidget.cpp
     Project              : LabPlot
     Description          : Datapicker
     --------------------------------------------------------------------
-    Copyright            : (C) 2016 by Ankit Wagadre (wagadre.ankit@gmail.com)
-    Copyright            : (C) 2016-2017 Alexander Semke (alexander.semke@web.de)
+    SPDX-FileCopyrightText: 2016-2022 Alexander Semke <alexander.semke@web.de>
 
- ***************************************************************************/
-/***************************************************************************
- *                                                                         *
- *  This program is free software; you can redistribute it and/or modify   *
- *  it under the terms of the GNU General Public License as published by   *
- *  the Free Software Foundation; either version 2 of the License, or      *
- *  (at your option) any later version.                                    *
- *                                                                         *
- *  This program is distributed in the hope that it will be useful,        *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
- *  GNU General Public License for more details.                           *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the Free Software           *
- *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
- *   Boston, MA  02110-1301  USA                                           *
- *                                                                         *
- ***************************************************************************/
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "ImportSQLDatabaseWidget.h"
 #include "DatabaseManagerDialog.h"
 #include "DatabaseManagerWidget.h"
 #include "backend/datasources/AbstractDataSource.h"
-#include "backend/datasources/filters/AbstractFileFilter.h"
 #include "backend/lib/macros.h"
 
 #include <QTimer>
@@ -45,16 +26,16 @@
 #include <KMessageBox>
 #include <KSharedConfig>
 #ifdef HAVE_KF5_SYNTAX_HIGHLIGHTING
-#include <KF5/KSyntaxHighlighting/SyntaxHighlighter>
-#include <KF5/KSyntaxHighlighting/Definition>
-#include <KF5/KSyntaxHighlighting/Theme>
+#include <KSyntaxHighlighting/SyntaxHighlighter>
+#include <KSyntaxHighlighting/Definition>
+#include <KSyntaxHighlighting/Theme>
 #endif
 
 ImportSQLDatabaseWidget::ImportSQLDatabaseWidget(QWidget* parent) : QWidget(parent) {
 	ui.setupUi(this);
 
 	ui.cbImportFrom->addItem(i18n("Table"));
-	ui.cbImportFrom->addItem(i18n("Custom query"));
+	ui.cbImportFrom->addItem(i18n("Custom Query"));
 
 	ui.bDatabaseManager->setIcon(QIcon::fromTheme("network-server-database"));
 	ui.bDatabaseManager->setToolTip(i18n("Manage connections"));
@@ -120,7 +101,10 @@ ImportSQLDatabaseWidget::ImportSQLDatabaseWidget(QWidget* parent) : QWidget(pare
 	                                       "<tr><td>dd.MM.yyyy</td><td>20.07.1969</td></tr>"
 	                                       "<tr><td>ddd MMMM d yy</td><td>Sun July 20 69</td></tr>"
 	                                       "<tr><td>'The day is' dddd</td><td>The day is Sunday</td></tr>"
-	                                       "</table>");
+	                                       "</table>"
+											"<br><br>"
+										   "In case the provided expression is empty, the format will be auto-detected."
+											);
 
 	ui.lDateTimeFormat->setToolTip(textDateTimeFormatShort);
 	ui.lDateTimeFormat->setWhatsThis(textDateTimeFormat);
@@ -137,11 +121,11 @@ ImportSQLDatabaseWidget::ImportSQLDatabaseWidget(QWidget* parent) : QWidget(pare
 
 	m_configPath = QStandardPaths::standardLocations(QStandardPaths::AppDataLocation).constFirst() +  "sql_connections";
 
-	connect( ui.cbConnection, SIGNAL(currentIndexChanged(int)), SLOT(connectionChanged()) );
-	connect( ui.cbImportFrom, SIGNAL(currentIndexChanged(int)), SLOT(importFromChanged(int)) );
-	connect( ui.bDatabaseManager, SIGNAL(clicked()), this, SLOT(showDatabaseManager()) );
-	connect( ui.lwTables, SIGNAL(currentRowChanged(int)), this, SLOT(refreshPreview()) );
-	connect( ui.bRefreshPreview, SIGNAL(clicked()), this, SLOT(refreshPreview()) );
+	connect(ui.cbConnection, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &ImportSQLDatabaseWidget::connectionChanged);
+	connect(ui.cbImportFrom, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &ImportSQLDatabaseWidget::importFromChanged);
+	connect(ui.bDatabaseManager, &QPushButton::clicked, this, &ImportSQLDatabaseWidget::showDatabaseManager);
+	connect(ui.lwTables, &QListWidget::currentRowChanged, this, &ImportSQLDatabaseWidget::refreshPreview);
+	connect(ui.bRefreshPreview, &QPushButton::clicked, this, &ImportSQLDatabaseWidget::refreshPreview);
 
 	//defer the loading of settings a bit in order to show the dialog prior to blocking the GUI in refreshPreview()
 	QTimer::singleShot( 100, this, SLOT(loadSettings()) );
@@ -164,9 +148,8 @@ void ImportSQLDatabaseWidget::loadSettings() {
 	int index = (decimalSeparator == '.') ? 0 : 1;
 	ui.cbDecimalSeparator->setCurrentIndex(config.readEntry("DecimalSeparator", index));
 
-	ui.cbDateTimeFormat->setCurrentItem(config.readEntry("DateTimeFormat", "yyyy-dd-MM hh:mm:ss:zzz"));
-	QList<int> defaultSizes;
-	defaultSizes << 100 << 100;
+	ui.cbDateTimeFormat->setCurrentText(config.readEntry("DateTimeFormat", "yyyy-dd-MM hh:mm:ss:zzz"));
+	QList<int> defaultSizes{100, 100};
 	ui.splitterMain->setSizes(config.readEntry("SplitterMainSizes", defaultSizes));
 	ui.splitterPreview->setSizes(config.readEntry("SplitterPreviewSizes", defaultSizes));
 	//TODO
@@ -200,7 +183,7 @@ QString ImportSQLDatabaseWidget::selectedTable() const {
 			return ui.lwTables->currentItem()->text();
 	}
 
-	return QString();
+	return {};
 }
 
 /*!
@@ -300,6 +283,9 @@ void ImportSQLDatabaseWidget::connectionChanged() {
 	} else
 		setInvalid();
 
+	//show the last used query
+	ui.teQuery->setText(group.readEntry("Query"));
+
 	RESET_CURSOR;
 }
 
@@ -312,6 +298,15 @@ void ImportSQLDatabaseWidget::refreshPreview() {
 	WAIT_CURSOR;
 	ui.twPreview->clear();
 
+	bool customQuery = (ui.cbImportFrom->currentIndex() != 0);
+
+	//save the last used custom query
+	if (customQuery) {
+		KConfig config(m_configPath, KConfig::SimpleConfig);
+		KConfigGroup group = config.group(ui.cbConnection->currentText());
+		group.writeEntry("Query", ui.teQuery->toPlainText());
+	}
+
 	//execute the current query (select on a table or a custom query)
 	const QString& query = currentQuery(true);
 	if (query.isEmpty()) {
@@ -348,7 +343,7 @@ void ImportSQLDatabaseWidget::refreshPreview() {
 		lang = QLocale::Language::C;
 	else
 		lang = QLocale::Language::German;
-	const QString& dateTimeFormat = ui.cbDateTimeFormat->currentText();
+	QString dateTimeFormat = ui.cbDateTimeFormat->currentText();
 // 	ui.twPreview->setRowCount(1); //add the first row for the check boxes
 	for (int i = 0; i < m_cols; ++i) {
 		//name
@@ -358,13 +353,13 @@ void ImportSQLDatabaseWidget::refreshPreview() {
 		const QString valueString = q.record().value(i).toString();
 		AbstractColumn::ColumnMode mode = AbstractFileFilter::columnMode(valueString, dateTimeFormat, lang);
 		m_columnModes << mode;
-		if (mode != AbstractColumn::ColumnMode::Numeric)
+		if (mode != AbstractColumn::ColumnMode::Double)
 			numeric = false;
 
 		//header item
-		QTableWidgetItem* item = new QTableWidgetItem(m_columnNames[i] + QLatin1String(" {") + ENUM_TO_STRING(AbstractColumn, ColumnMode, mode) + QLatin1String("}"));
+		auto* item = new QTableWidgetItem(m_columnNames[i] + QLatin1String(" {") + ENUM_TO_STRING(AbstractColumn, ColumnMode, mode) + QLatin1String("}"));
 		item->setTextAlignment(Qt::AlignLeft);
-		item->setIcon(AbstractColumn::iconForMode(mode));
+		item->setIcon(AbstractColumn::modeIcon(mode));
 		ui.twPreview->setHorizontalHeaderItem(i, item);
 
 		//create checked items
@@ -374,7 +369,6 @@ void ImportSQLDatabaseWidget::refreshPreview() {
 	}
 
 	//preview the data
-	const bool customQuery = (ui.cbImportFrom->currentIndex() != 0);
 	int row = 0;
 	do {
 		for (int col = 0; col < m_cols; ++col) {
@@ -394,7 +388,7 @@ void ImportSQLDatabaseWidget::refreshPreview() {
 
 	if (numeric != m_numeric) {
 		m_numeric = numeric;
-		emit stateChanged();
+		Q_EMIT stateChanged();
 	}
 
 	RESET_CURSOR;
@@ -404,10 +398,12 @@ void ImportSQLDatabaseWidget::importFromChanged(int index) {
 	if (index == 0) { //import from a table
 		ui.gbQuery->hide();
 		ui.lwTables->show();
+		ui.bRefreshPreview->setToolTip(i18n("Refresh the data preview of the selected table"));
 	} else { //import the result set of a custom query
 		ui.gbQuery->show();
 		ui.lwTables->hide();
 		ui.twPreview->clear();
+		ui.bRefreshPreview->setToolTip(i18n("Execute the query and preview its result"));
 	}
 
 	refreshPreview();
@@ -442,7 +438,7 @@ void ImportSQLDatabaseWidget::read(AbstractDataSource* dataSource, AbstractFileF
 	int columnOffset = dataSource->prepareImport(dataContainer, importMode, rows, m_cols, m_columnNames, m_columnModes);
 
 	//number and DateTime formatting
-	const QString& dateTimeFormat = ui.cbDateTimeFormat->currentText();
+	QString dateTimeFormat = ui.cbDateTimeFormat->currentText();
 
 	//TODO: use general setting for decimal separator?
 	QLocale::Language lang;
@@ -460,7 +456,7 @@ void ImportSQLDatabaseWidget::read(AbstractDataSource* dataSource, AbstractFileF
 
 			// set value depending on data type
 			switch (m_columnModes[col]) {
-			case AbstractColumn::ColumnMode::Numeric: {
+			case AbstractColumn::ColumnMode::Double: {
 				bool isNumber;
 				const double value = numberFormat.toDouble(valueString, &isNumber);
 				static_cast<QVector<double>*>(dataContainer[col])->operator[](row) = (isNumber ? value : NAN);
@@ -493,7 +489,7 @@ void ImportSQLDatabaseWidget::read(AbstractDataSource* dataSource, AbstractFileF
 		}
 
 		row++;
-		emit completed(100 * row/rows);
+		Q_EMIT completed(100 * row/rows);
 	} while (q.next());
 	DEBUG("	Read " << row << " rows");
 
@@ -539,7 +535,7 @@ QString ImportSQLDatabaseWidget::currentQuery(bool preview) {
 	The selected connection is selected in the connection combo box in this widget.
 **/
 void ImportSQLDatabaseWidget::showDatabaseManager() {
-	DatabaseManagerDialog* dlg = new DatabaseManagerDialog(this, ui.cbConnection->currentText());
+	auto* dlg = new DatabaseManagerDialog(this, ui.cbConnection->currentText());
 
 	if (dlg->exec() == QDialog::Accepted) {
 		//re-read the available connections to be in sync with the changes in DatabaseManager
@@ -564,13 +560,13 @@ void ImportSQLDatabaseWidget::setInvalid() {
 		ui.twPreview->setRowCount(0);
 
 		m_valid = false;
-		emit stateChanged();
+		Q_EMIT stateChanged();
 	}
 }
 
 void ImportSQLDatabaseWidget::setValid() {
 	if (!m_valid) {
 		m_valid = true;
-		emit stateChanged();
+		Q_EMIT stateChanged();
 	}
 }
