@@ -1,30 +1,12 @@
-/***************************************************************************
+/*
     File                 : ImageDock.cpp
     Project              : LabPlot
     Description          : widget for image properties
     --------------------------------------------------------------------
-    Copyright            : (C) 2019-2020 by Alexander Semke (alexander.semke@web.de)
+    SPDX-FileCopyrightText: 2019-2020 Alexander Semke <alexander.semke@web.de>
 
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *  This program is free software; you can redistribute it and/or modify   *
- *  it under the terms of the GNU General Public License as published by   *
- *  the Free Software Foundation; either version 2 of the License, or      *
- *  (at your option) any later version.                                    *
- *                                                                         *
- *  This program is distributed in the hope that it will be useful,        *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
- *  GNU General Public License for more details.                           *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the Free Software           *
- *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
- *   Boston, MA  02110-1301  USA                                           *
- *                                                                         *
- ***************************************************************************/
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "ImageDock.h"
 #include "backend/worksheet/Image.h"
@@ -35,14 +17,10 @@
 
 #include <QCompleter>
 #include <QDirModel>
-#include <QFileDialog>
-#include <QImageReader>
 #include <QPageSize>
 
 #include <KConfig>
-#include <KConfigGroup>
 #include <KLocalizedString>
-#include <KSharedConfig>
 
 /*!
   \class ImageDock
@@ -54,7 +32,8 @@
 ImageDock::ImageDock(QWidget* parent) : BaseDock(parent) {
 	ui.setupUi(this);
 	m_leName = ui.leName;
-	m_leComment = ui.leComment;
+	m_teComment = ui.teComment;
+	m_teComment->setFixedHeight(1.2 * m_leName->height());
 
 	ui.bOpen->setIcon( QIcon::fromTheme("document-open") );
 	ui.leFileName->setCompleter(new QCompleter(new QDirModel, this));
@@ -66,12 +45,10 @@ ImageDock::ImageDock(QWidget* parent) : BaseDock(parent) {
 	ui.cbPositionX->addItem(i18n("Left"));
 	ui.cbPositionX->addItem(i18n("Center"));
 	ui.cbPositionX->addItem(i18n("Right"));
-	ui.cbPositionX->addItem(i18n("Custom"));
 
 	ui.cbPositionY->addItem(i18n("Top"));
 	ui.cbPositionY->addItem(i18n("Center"));
 	ui.cbPositionY->addItem(i18n("Bottom"));
-	ui.cbPositionY->addItem(i18n("Custom"));
 
 	ui.cbHorizontalAlignment->addItem(i18n("Left"));
 	ui.cbHorizontalAlignment->addItem(i18n("Center"));
@@ -100,12 +77,13 @@ ImageDock::ImageDock(QWidget* parent) : BaseDock(parent) {
 	ui.cbBorderStyle->addItem(i18n("Dash dot line"));
 	ui.cbBorderStyle->addItem(i18n("Dash dot dot line"));
 
-	updateLocale();
+	ImageDock::updateLocale();
 
 	//SLOTs
 	//General
 	connect(ui.leName, &QLineEdit::textChanged, this, &ImageDock::nameChanged);
-	connect(ui.leComment, &QLineEdit::textChanged, this, &ImageDock::commentChanged);
+	connect(ui.chbEmbedded, &QCheckBox::clicked, this, &ImageDock::embeddedChanged);
+	connect(ui.teComment, &QTextEdit::textChanged, this, &ImageDock::commentChanged);
 	connect(ui.bOpen, &QPushButton::clicked, this, &ImageDock::selectFile);
 	connect(ui.leFileName, &QLineEdit::returnPressed, this, &ImageDock::fileNameChanged);
 	connect(ui.leFileName, &QLineEdit::textChanged, this, &ImageDock::fileNameChanged);
@@ -152,18 +130,18 @@ void ImageDock::setImages(QList<Image*> list) {
 		ui.lName->setEnabled(true);
 		ui.leName->setEnabled(true);
 		ui.lComment->setEnabled(true);
-		ui.leComment->setEnabled(true);
+		ui.teComment->setEnabled(true);
 
 		ui.leName->setText(m_image->name());
-		ui.leComment->setText(m_image->comment());
+		ui.teComment->setText(m_image->comment());
 	} else {
 		ui.lName->setEnabled(false);
 		ui.leName->setEnabled(false);
 		ui.lComment->setEnabled(false);
-		ui.leComment->setEnabled(false);
+		ui.teComment->setEnabled(false);
 
 		ui.leName->setText(QString());
-		ui.leComment->setText(QString());
+		ui.teComment->setText(QString());
 	}
 	ui.leName->setStyleSheet("");
 	ui.leName->setToolTip("");
@@ -173,7 +151,9 @@ void ImageDock::setImages(QList<Image*> list) {
 
 	//init connections
 	//General
+	connect(m_image, &Image::aspectDescriptionChanged, this, &ImageDock::aspectDescriptionChanged);
 	connect(m_image, &Image::fileNameChanged, this, &ImageDock::imageFileNameChanged);
+	connect(m_image, &Image::embeddedChanged, this, &ImageDock::imageEmbeddedChanged);
 	connect(m_image, &Image::opacityChanged, this, &ImageDock::imageOpacityChanged);
 	connect(m_image, &Image::visibleChanged, this, &ImageDock::imageVisibleChanged);
 
@@ -241,45 +221,49 @@ void ImageDock::updateUnits() {
 //*************************************************************
 //******** SLOTs for changes triggered in ImageDock ***********
 //*************************************************************
-
 /*!
 	opens a file dialog and lets the user select the image file.
 */
 void ImageDock::selectFile() {
-	KConfigGroup conf(KSharedConfig::openConfig(), "ImageDock");
-	QString dir = conf.readEntry("LastImageDir", "");
-
-	QString formats;
-	for (const QByteArray& format : QImageReader::supportedImageFormats()) {
-		QString f = "*." + QString(format.constData());
-		if (f == QLatin1String("*.svg"))
-			continue;
-		formats.isEmpty() ? formats += f : formats += ' ' + f;
-	}
-
-	QString path = QFileDialog::getOpenFileName(this, i18n("Select the image file"), dir, i18n("Images (%1)", formats));
+	const QString& path = GuiTools::openImageFile(QLatin1String("ImageDock"));
 	if (path.isEmpty())
-		return; //cancel was clicked in the file-dialog
-
-	int pos = path.lastIndexOf(QLatin1String("/"));
-	if (pos != -1) {
-		QString newDir = path.left(pos);
-		if (newDir != dir)
-			conf.writeEntry("LastImageDir", newDir);
-	}
+		return;
 
 	ui.leFileName->setText(path);
+
+	//above the path was set in the text field which triggered setting
+	//of it in Image and loading of the image. Call embeddedChanged()
+	//to update the text field and to show the actual file name only
+	//and not the whole path if the image is being embedded.
+	Lock lock(m_initializing);
+	embeddedChanged(ui.chbEmbedded->checkState());
+}
+
+void ImageDock::embeddedChanged(int state) {
+	bool embedded = static_cast<bool>(state);
+	if (embedded) {
+		QFileInfo fi(m_image->fileName());
+		ui.leFileName->setText(fi.fileName());
+		ui.leFileName->setEnabled(false);
+	} else {
+		ui.leFileName->setText(m_image->fileName());
+		ui.leFileName->setEnabled(true);
+	}
+
+	if (m_initializing)
+		return;
+
+	for (auto* image : m_imageList)
+		image->setEmbedded(embedded);
 }
 
 void ImageDock::fileNameChanged() {
 	if (m_initializing)
 		return;
 
-	QString fileName = ui.leFileName->text();
-	if (!fileName.isEmpty() && !QFile::exists(fileName))
-		ui.leFileName->setStyleSheet("QLineEdit{background:red;}");
-	else
-		ui.leFileName->setStyleSheet(QString());
+	const QString& fileName = ui.leFileName->text();
+	bool invalid = (!fileName.isEmpty() && !QFile::exists(fileName));
+	GuiTools::highlight(ui.leFileName, invalid);
 
 	for (auto* image : m_imageList)
 		image->setFileName(fileName);
@@ -295,8 +279,7 @@ void ImageDock::opacityChanged(int value) {
 }
 
 //Size
-void ImageDock::sizeChanged(int index) {
-	Q_UNUSED(index);
+void ImageDock::sizeChanged(int /*index*/) {
 }
 
 void ImageDock::widthChanged(double value) {
@@ -330,12 +313,6 @@ void ImageDock::keepRatioChanged(int state) {
     called when label's current horizontal position relative to its parent (left, center, right, custom ) is changed.
 */
 void ImageDock::positionXChanged(int index) {
-	//Enable/disable the spinbox for the x- oordinates if the "custom position"-item is selected/deselected
-	if (index == ui.cbPositionX->count()-1 )
-		ui.sbPositionX->setEnabled(true);
-	else
-		ui.sbPositionX->setEnabled(false);
-
 	if (m_initializing)
 		return;
 
@@ -349,16 +326,10 @@ void ImageDock::positionXChanged(int index) {
     called when label's current horizontal position relative to its parent (top, center, bottom, custom ) is changed.
 */
 void ImageDock::positionYChanged(int index) {
-	//Enable/disable the spinbox for the y-coordinates if the "custom position"-item is selected/deselected
-	if (index == ui.cbPositionY->count()-1 )
-		ui.sbPositionY->setEnabled(true);
-	else
-		ui.sbPositionY->setEnabled(false);
-
 	if (m_initializing)
 		return;
 
-	WorksheetElement::PositionWrapper position = m_image->position();
+	auto position = m_image->position();
 	position.verticalPosition = WorksheetElement::VerticalPosition(index);
 	for (auto* image : m_imageList)
 		image->setPosition(position);
@@ -368,20 +339,24 @@ void ImageDock::customPositionXChanged(double value) {
 	if (m_initializing)
 		return;
 
-	WorksheetElement::PositionWrapper position = m_image->position();
-	position.point.setX(Worksheet::convertToSceneUnits(value, m_worksheetUnit));
-	for (auto* image : m_imageList)
+	const double x = Worksheet::convertToSceneUnits(value, m_worksheetUnit);
+	for (auto* image : m_imageList) {
+		auto position = image->position();
+		position.point.setX(x);
 		image->setPosition(position);
+	}
 }
 
 void ImageDock::customPositionYChanged(double value) {
 	if (m_initializing)
 		return;
 
-	WorksheetElement::PositionWrapper position = m_image->position();
-	position.point.setY(Worksheet::convertToSceneUnits(value, m_worksheetUnit));
-	for (auto* image : m_imageList)
+	const double y = Worksheet::convertToSceneUnits(value, m_worksheetUnit);
+	for (auto* image : m_imageList) {
+		auto position = image->position();
+		position.point.setY(y);
 		image->setPosition(position);
+	}
 }
 
 void ImageDock::horizontalAlignmentChanged(int index) {
@@ -470,24 +445,18 @@ void ImageDock::borderOpacityChanged(int value) {
 //*************************************************************
 //********** SLOTs for changes triggered in Image *************
 //*************************************************************
-void ImageDock::imageDescriptionChanged(const AbstractAspect* aspect) {
-	if (m_image != aspect)
-		return;
-
-	m_initializing = true;
-	if (aspect->name() != ui.leName->text())
-		ui.leName->setText(aspect->name());
-	else if (aspect->comment() != ui.leComment->text())
-		ui.leComment->setText(aspect->comment());
-	m_initializing = false;
-}
-
 void ImageDock::imageFileNameChanged(const QString& name) {
 	m_initializing = true;
 	ui.leFileName->setText(name);
 	m_initializing = false;
 }
 
+void ImageDock::imageEmbeddedChanged(bool keep) {
+	m_initializing = true;
+	ui.chbEmbedded->setChecked(keep);
+	m_initializing = false;
+}
+
 void ImageDock::imageOpacityChanged(float opacity) {
 	m_initializing = true;
 	ui.sbOpacity->setValue( qRound(opacity*100.0) );
@@ -507,7 +476,6 @@ void ImageDock::imageHeightChanged(int height) {
 	m_initializing = false;
 }
 
-
 void ImageDock::imageKeepRatioChanged(bool keep) {
 	m_initializing = true;
 	ui.chbKeepRatio->setChecked(keep);
@@ -574,9 +542,11 @@ void ImageDock::load() {
 	if (!m_image)
 		return;
 
-	m_initializing = true;
+	Lock lock(m_initializing);
 
 	ui.leFileName->setText(m_image->fileName());
+	ui.chbEmbedded->setChecked(m_image->embedded());
+	embeddedChanged(ui.chbEmbedded->checkState());
 	ui.chbVisible->setChecked(m_image->isVisible());
 
 	//Size
@@ -602,6 +572,4 @@ void ImageDock::load() {
 	ui.sbBorderWidth->setValue( Worksheet::convertFromSceneUnits(m_image->borderPen().widthF(), Worksheet::Unit::Point) );
 	ui.sbBorderOpacity->setValue( round(m_image->borderOpacity()*100) );
 	GuiTools::updatePenStyles(ui.cbBorderStyle, ui.kcbBorderColor->color());
-
-	m_initializing = false;
 }
