@@ -1,33 +1,18 @@
-/***************************************************************************
-File                 : LiveDataDock.cpp
-Project              : LabPlot
-Description          : Dock widget for live data properties
---------------------------------------------------------------------
-Copyright            : (C) 2017 by Fabian Kristof (fkristofszabolcs@gmail.com)
-Copyright            : (C) 2018-2019 Kovacs Ferencz (kferike98@gmail.com)
-Copyright            : (C) 2018 by Stefan Gerlach (stefan.gerlach@uni.kn)
-Copyright            : (C) 2017-2019 Alexander Semke (alexander.semke@web.de)
-***************************************************************************/
+/*
+    File                 : LiveDataDock.cpp
+    Project              : LabPlot
+    Description          : Dock widget for live data properties
+    --------------------------------------------------------------------
+    SPDX-FileCopyrightText: 2017 Fabian Kristof <fkristofszabolcs@gmail.com>
+    SPDX-FileCopyrightText: 2018-2019 Kovacs Ferencz <kferike98@gmail.com>
+    SPDX-FileCopyrightText: 2018 Stefan Gerlach <stefan.gerlach@uni.kn>
+    SPDX-FileCopyrightText: 2017-2020 Alexander Semke <alexander.semke@web.de>
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
-/***************************************************************************
-*                                                                         *
-*  This program is free software; you can redistribute it and/or modify   *
-*  it under the terms of the GNU General Public License as published by   *
-*  the Free Software Foundation; either version 2 of the License, or      *
-*  (at your option) any later version.                                    *
-*                                                                         *
-*  This program is distributed in the hope that it will be useful,        *
-*  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
-*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
-*  GNU General Public License for more details.                           *
-*                                                                         *
-*   You should have received a copy of the GNU General Public License     *
-*   along with this program; if not, write to the Free Software           *
-*   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
-*   Boston, MA  02110-1301  USA                                           *
-*                                                                         *
-***************************************************************************/
 #include "LiveDataDock.h"
+#include "kdefrontend/GuiTools.h"
+
 #include <QCompleter>
 #include <QFile>
 #include <QStandardItemModel>
@@ -103,15 +88,12 @@ LiveDataDock::~LiveDataDock() = default;
  */
 void LiveDataDock::setMQTTClient(MQTTClient* const client) {
 	m_liveDataSource = nullptr; // prevent updates due to changes to input widgets
-	if (m_mqttClient == client)
-		return;
 	auto oldclient = m_mqttClient;
 	m_mqttClient = nullptr; // prevent updates due to changes to input widgets
+	const QPair<QString, quint16> id(client->clientHostName(), client->clientPort());
 
 	ui.leName->setText(client->name());
-	const QPair<QString, quint16> id(client->clientHostName(), client->clientPort());
 	ui.leSourceInfo->setText(QStringLiteral("%1:%2").arg(id.first).arg(id.second));
-
 	ui.sbUpdateInterval->setValue(client->updateInterval());
 	ui.cbUpdateType->setCurrentIndex(static_cast<int>(client->updateType()));
 	ui.cbReadingType->setCurrentIndex(static_cast<int>(client->readingType()));
@@ -121,7 +103,8 @@ void LiveDataDock::setMQTTClient(MQTTClient* const client) {
 		ui.sbUpdateInterval->hide();
 	}
 
-	if (client->isPaused()) {
+	m_paused = client->isPaused();
+	if (m_paused) {
 		ui.bPausePlayReading->setText(i18n("Continue reading"));
 		ui.bPausePlayReading->setIcon(QIcon::fromTheme(QLatin1String("media-record")));
 	} else {
@@ -139,7 +122,7 @@ void LiveDataDock::setMQTTClient(MQTTClient* const client) {
 		ui.sbSampleSize->setValue(client->sampleSize());
 
 	// disable "whole file" option
-	const QStandardItemModel* model = qobject_cast<const QStandardItemModel*>(ui.cbReadingType->model());
+	const auto* model = qobject_cast<const QStandardItemModel*>(ui.cbReadingType->model());
 	QStandardItem* item = model->item(static_cast<int>(LiveDataSource::ReadingType::WholeFile));
 	item->setFlags(item->flags() & ~(Qt::ItemIsSelectable | Qt::ItemIsEnabled));
 	if (ui.cbReadingType->currentIndex() == static_cast<int>(LiveDataSource::ReadingType::WholeFile))
@@ -156,6 +139,7 @@ void LiveDataDock::setMQTTClient(MQTTClient* const client) {
 	ui.bLWT->show();
 
 	m_previousHost = m_currentHost;
+
 	//if there isn't a client with this hostname we instantiate a new one
 	auto it = m_hosts.find(id);
 	if (it == m_hosts.end()) {
@@ -173,7 +157,6 @@ void LiveDataDock::setMQTTClient(MQTTClient* const client) {
 		connect(m_subscriptionWidget, &MQTTSubscriptionWidget::removeMQTTSubscription, client, &MQTTClient::removeMQTTSubscription);
 		connect(m_subscriptionWidget, &MQTTSubscriptionWidget::makeSubscription, client, &MQTTClient::addMQTTSubscription);
 
-
 		m_currentHost->client->setHostname(id.first);
 		m_currentHost->client->setPort(id.second);
 
@@ -193,14 +176,14 @@ void LiveDataDock::setMQTTClient(MQTTClient* const client) {
 
 	if (m_previousMQTTClient == nullptr) {
 		m_updateSubscriptionConn = connect(client, &MQTTClient::MQTTSubscribed, [this]() {
-			emit updateSubscriptionTree(m_mqttClient->MQTTSubscriptions());
+			Q_EMIT updateSubscriptionTree(m_mqttClient->MQTTSubscriptions());
 		});
 
 		//Fill the subscription tree(useful if the MQTTClient was loaded)
 		QVector<QString> topics = client->topicNames();
 		for (const auto& topic : topics)
 			addTopicToTree(topic);
-		emit updateSubscriptionTree(m_mqttClient->MQTTSubscriptions());
+		Q_EMIT updateSubscriptionTree(m_mqttClient->MQTTSubscriptions());
 	}
 
 	//if the previous MQTTClient's host name was different from the current one we have to disconnect some slots
@@ -220,16 +203,16 @@ void LiveDataDock::setMQTTClient(MQTTClient* const client) {
 		m_previousHost->topicList = m_subscriptionWidget->getTopicList();
 		m_subscriptionWidget->setTopicList(m_currentHost->topicList);
 
-		emit MQTTClearTopics();
+		Q_EMIT MQTTClearTopics();
 		//repopulating the tree widget with the already known topics of the client
-		for (int i = 0; i < m_currentHost->addedTopics.size(); ++i)
-			addTopicToTree(m_currentHost->addedTopics.at(i));
+		for (const auto& topic : m_currentHost->addedTopics)
+			addTopicToTree(topic);
 
 		//fill subscriptions tree widget
-		emit updateSubscriptionTree(m_mqttClient->MQTTSubscriptions());
+		Q_EMIT updateSubscriptionTree(m_mqttClient->MQTTSubscriptions());
 
 		m_updateSubscriptionConn = connect(client, &MQTTClient::MQTTSubscribed, [this]() {
-			emit updateSubscriptionTree(m_mqttClient->MQTTSubscriptions());
+			Q_EMIT updateSubscriptionTree(m_mqttClient->MQTTSubscriptions());
 		});
 		connect(m_currentHost->client, &QMqttClient::messageReceived, this, &LiveDataDock::mqttMessageReceived);
 
@@ -254,8 +237,6 @@ void LiveDataDock::setLiveDataSource(LiveDataSource* const source) {
 #ifdef HAVE_MQTT
 	m_mqttClient = nullptr;
 #endif
-// 	if (m_liveDataSource == source)
-// 		return;
 	m_liveDataSource = nullptr; // prevent updates due to changes to input widgets
 
 	ui.leName->setText(source->name());
@@ -270,13 +251,12 @@ void LiveDataDock::setLiveDataSource(LiveDataSource* const source) {
 	ui.cbReadingType->setCurrentIndex(static_cast<int>(readingType));
 
 	switch (sourceType) {
-	case LiveDataSource::SourceType::FileOrPipe:
+	case LiveDataSource::SourceType::FileOrPipe: {
 		ui.leSourceInfo->setText(source->fileName());
-		if (QFile::exists(source->fileName()))
-			ui.leSourceInfo->setStyleSheet(QString());
-		else
-			ui.leSourceInfo->setStyleSheet("QLineEdit{background:red;}");
+		bool invalid = !QFile::exists(source->fileName());
+		GuiTools::highlight(ui.leSourceInfo, invalid);
 		break;
+	}
 	case LiveDataSource::SourceType::NetworkTcpSocket:
 	case LiveDataSource::SourceType::NetworkUdpSocket:
 		ui.leSourceInfo->setText(QStringLiteral("%1:%2").arg(source->host()).arg(source->port()));
@@ -296,7 +276,8 @@ void LiveDataDock::setLiveDataSource(LiveDataSource* const source) {
 		ui.sbUpdateInterval->hide();
 	}
 
-	if (source->isPaused()) {
+	m_paused = source->isPaused();
+	if (m_paused) {
 		ui.bPausePlayReading->setText(i18n("Continue Reading"));
 		ui.bPausePlayReading->setIcon(QIcon::fromTheme(QLatin1String("media-record")));
 	} else {
@@ -381,7 +362,7 @@ void LiveDataDock::updateNow() {
 void LiveDataDock::nameChanged(const QString& name) {
 	if (m_liveDataSource) {
 		if (!m_liveDataSource->setName(name, false)) {
-			ui.leName->setStyleSheet("background:red;");
+			SET_WARNING_STYLE(m_leName)
 			ui.leName->setToolTip(i18n("Please choose another name, because this is already in use."));
 			return;
 		}
@@ -389,7 +370,7 @@ void LiveDataDock::nameChanged(const QString& name) {
 #ifdef HAVE_MQTT
 	else if (m_mqttClient) {
 		if (!m_mqttClient->setName(name, false)) {
-			ui.leName->setStyleSheet("background:red;");
+			SET_WARNING_STYLE(m_leName)
 			ui.leName->setToolTip(i18n("Please choose another name, because this is already in use."));
 			return;
 		}
@@ -478,7 +459,7 @@ void LiveDataDock::readingTypeChanged(int idx) {
 	}
 #ifdef HAVE_MQTT
 	else if (m_mqttClient) {
-		MQTTClient::ReadingType type = static_cast<MQTTClient::ReadingType>(idx);
+		auto type = static_cast<MQTTClient::ReadingType>(idx);
 
 		if (type == MQTTClient::ReadingType::TillEnd) {
 			ui.lSampleSize->hide();
@@ -569,13 +550,10 @@ void LiveDataDock::pauseContinueReading() {
  * \param state the state of the checbox
  */
 void LiveDataDock::useWillMessage(bool use) {
-	qDebug()<<"Use will message: " << use;
-
 	if (use) {
 		m_mqttClient->setMQTTWillUse(true);
 		if (m_mqttClient->willUpdateType() == MQTTClient::WillUpdateType::OnClick)
 			ui.bWillUpdateNow->show();
-
 	} else {
 		m_mqttClient->setMQTTWillUse(false);
 		ui.bWillUpdateNow->hide();
@@ -704,8 +682,7 @@ void LiveDataDock::onMQTTConnect() {
  *\brief called when the client receives a message
  * if the message arrived from a new topic, the topic is put in twTopics
  */
-void LiveDataDock::mqttMessageReceived(const QByteArray& message, const QMqttTopicName& topic) {
-	Q_UNUSED(message)
+void LiveDataDock::mqttMessageReceived(const QByteArray& /*message*/, const QMqttTopicName& topic) {
 	if (!m_currentHost->addedTopics.contains(topic.name())) {
 		m_currentHost->addedTopics.push_back(topic.name());
 		addTopicToTree(topic.name());
@@ -722,7 +699,11 @@ void LiveDataDock::addTopicToTree(const QString &topicName) {
 	QChar sep = '/';
 	QString rootName;
 	if (topicName.contains(sep)) {
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+		QStringList list = topicName.split(sep, Qt::SkipEmptyParts);
+#else
 		QStringList list = topicName.split(sep, QString::SkipEmptyParts);
+#endif
 
 		if (!list.isEmpty()) {
 			rootName = list.at(0);
@@ -786,24 +767,27 @@ void LiveDataDock::addTopicToTree(const QString &topicName) {
 
 	//if a subscribed topic contains the new topic, we have to update twSubscriptions
 	for (int i = 0; i < m_subscriptionWidget->subscriptionCount(); ++i) {
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+		QStringList subscriptionName = m_subscriptionWidget->topLevelSubscription(i)->text(0).split('/', Qt::SkipEmptyParts);
+#else
 		QStringList subscriptionName = m_subscriptionWidget->topLevelSubscription(i)->text(0).split('/', QString::SkipEmptyParts);
+#endif
 		if (rootName == subscriptionName[0]) {
-			emit updateSubscriptionTree(m_mqttClient->MQTTSubscriptions());
+			Q_EMIT updateSubscriptionTree(m_mqttClient->MQTTSubscriptions());
 			break;
 		}
 	}
 
 	//signals that a newTopic was added, in order to fill the completer of leTopics
 	//we have to pass the whole topic name, not just the root name, for testing purposes
-	emit newTopic(topicName);
+	Q_EMIT newTopic(topicName);
 }
 
 /*!
  *\brief called when a client receives a message, if the clients hostname isn't identic with the host name of MQTTClient
  * if the message arrived from a new topic, the topic is added to the host data
  */
-void LiveDataDock::mqttMessageReceivedInBackground(const QByteArray& message, const QMqttTopicName& topic) {
-	Q_UNUSED(message)
+void LiveDataDock::mqttMessageReceivedInBackground(const QByteArray& /*message*/, const QMqttTopicName& topic) {
 	if (!m_currentHost->addedTopics.contains(topic.name()))
 		m_currentHost->addedTopics.push_back(topic.name());
 }
@@ -829,13 +813,13 @@ void LiveDataDock::removeClient(const QString& hostname, quint16 port) {
 
 	host.client->disconnectFromHost();
 
-	if (m_previousMQTTClient != nullptr && m_previousMQTTClient->clientHostName() == hostname) {
+	if (m_previousMQTTClient && m_previousMQTTClient->clientHostName() == hostname) {
 		disconnect(m_previousHost->client, &QMqttClient::messageReceived, this, &LiveDataDock::mqttMessageReceivedInBackground);
 		m_previousMQTTClient = nullptr;
 	}
 
-	if (m_mqttClient->clientHostName() == hostname) {
-		emit MQTTClearTopics();
+	if (m_mqttClient && m_mqttClient->clientHostName() == hostname) {
+		Q_EMIT MQTTClearTopics();
 		m_mqttClient = nullptr;
 	}
 
@@ -848,7 +832,11 @@ void LiveDataDock::removeClient(const QString& hostname, quint16 port) {
  * \param topic
  */
 bool LiveDataDock::testSubscribe(const QString& topic) {
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+	QStringList topicList = topic.split('/', Qt::SkipEmptyParts);
+#else
 	QStringList topicList = topic.split('/', QString::SkipEmptyParts);
+#endif
 	QTreeWidgetItem* currentItem = nullptr;
 	for (int i = 0; i < m_subscriptionWidget->topicCount(); ++i) {
 		if (m_subscriptionWidget->topLevelTopic(i)->text(0) == topicList[0]) {
@@ -931,7 +919,7 @@ void LiveDataDock::showWillSettings() {
 		menu.close();
 	});
 
-	QWidgetAction* widgetAction = new QWidgetAction(this);
+	auto* widgetAction = new QWidgetAction(this);
 	widgetAction->setDefaultWidget(&willSettingsWidget);
 	menu.addAction(widgetAction);
 
