@@ -1,38 +1,26 @@
-/***************************************************************************
+/*
     File                 : PlotDataDialog.cpp
     Project              : LabPlot
     Description          : Dialog for generating plots for the spreadsheet data
     --------------------------------------------------------------------
-    Copyright            : (C) 2017-2019 by Alexander Semke (alexander.semke@web.de)
+	SPDX-FileCopyrightText: 2017-2022 Alexander Semke <alexander.semke@web.de>
+    SPDX-FileCopyrightText: 2022 Stefan Gerlach <stefan.gerlach@uni.kn>
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *  This program is free software; you can redistribute it and/or modify   *
- *  it under the terms of the GNU General Public License as published by   *
- *  the Free Software Foundation; either version 2 of the License, or      *
- *  (at your option) any later version.                                    *
- *                                                                         *
- *  This program is distributed in the hope that it will be useful,        *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
- *  GNU General Public License for more details.                           *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the Free Software           *
- *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
- *   Boston, MA  02110-1301  USA                                           *
- *                                                                         *
- ***************************************************************************/
 
 #include "PlotDataDialog.h"
 #include "backend/core/AspectTreeModel.h"
 #include "backend/core/Project.h"
 #include "backend/core/column/Column.h"
-#include "backend/datapicker/DatapickerCurve.h"
+#include "backend/lib/Range.h"
 #include "backend/spreadsheet/Spreadsheet.h"
+#include "backend/worksheet/TextLabel.h"
+#include "backend/worksheet/Worksheet.h"
 #include "backend/worksheet/plots/cartesian/Axis.h"
+#include "backend/worksheet/plots/cartesian/BoxPlot.h"
+#include "backend/worksheet/plots/cartesian/CartesianPlot.h"
+#include "backend/worksheet/plots/cartesian/Histogram.h"
 #include "backend/worksheet/plots/cartesian/XYAnalysisCurve.h"
 #include "backend/worksheet/plots/cartesian/XYCurve.h"
 #include "backend/worksheet/plots/cartesian/XYDataReductionCurve.h"
@@ -42,17 +30,13 @@
 #include "backend/worksheet/plots/cartesian/XYSmoothCurve.h"
 #include "backend/worksheet/plots/cartesian/XYFitCurve.h"
 #include "backend/worksheet/plots/cartesian/XYFourierFilterCurve.h"
+#include "commonfrontend/spreadsheet/SpreadsheetView.h"
+#include "commonfrontend/widgets/TreeViewComboBox.h"
 
 #ifdef HAVE_MQTT
 #include "backend/datasources/MQTTTopic.h"
 #endif
 
-#include "backend/worksheet/plots/cartesian/CartesianPlot.h"
-#include "backend/worksheet/Worksheet.h"
-#include "backend/worksheet/TextLabel.h"
-#include "commonfrontend/spreadsheet/SpreadsheetView.h"
-#include "commonfrontend/widgets/TreeViewComboBox.h"
-
 #include <QDialogButtonBox>
 #include <QPushButton>
 #include <QWindow>
@@ -80,10 +64,10 @@ PlotDataDialog::PlotDataDialog(Spreadsheet* s, PlotType type, QWidget* parent) :
 	setWindowTitle(i18nc("@title:window", "Plot Spreadsheet Data"));
 	setWindowIcon(QIcon::fromTheme("office-chart-line"));
 
-	QWidget* mainWidget = new QWidget(this);
+	auto* mainWidget = new QWidget(this);
 	ui->setupUi(mainWidget);
 
-	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
+	auto* buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
 	m_okButton = buttonBox->button(QDialogButtonBox::Ok);
 	m_okButton->setDefault(true);
 	m_okButton->setToolTip(i18n("Plot the selected data"));
@@ -195,7 +179,7 @@ PlotDataDialog::~PlotDataDialog() {
 	delete m_worksheetsModel;
 }
 
-void PlotDataDialog::setAnalysisAction(AnalysisAction action) {
+void PlotDataDialog::setAnalysisAction(XYAnalysisCurve::AnalysisAction action) {
 	m_analysisAction = action;
 	m_analysisMode = true;
 	ui->spacer->changeSize(0, 40);
@@ -256,6 +240,7 @@ void PlotDataDialog::processColumns() {
 		processColumnsForXYCurve(columnNames, xColumnName);
 		break;
 	case PlotType::Histogram:
+	case PlotType::BoxPlot:
 		processColumnsForHistogram(columnNames);
 		break;
 	}
@@ -294,7 +279,7 @@ void PlotDataDialog::processColumnsForXYCurve(const QStringList& columnNames, co
 	}
 
 	//show all selected/available column names in the data comboboxes
-	for (QComboBox* const comboBox : m_columnComboBoxes)
+	for (auto* const comboBox : m_columnComboBoxes)
 		comboBox->addItems(columnNames);
 
 	if (!xColumnName.isEmpty()) {
@@ -323,8 +308,16 @@ void PlotDataDialog::processColumnsForXYCurve(const QStringList& columnNames, co
 	}
 }
 
+/*!
+ * processes columns for cases where one single column
+ * is required per "plot" (histogram, boxplot, etc.)
+ * */
 void PlotDataDialog::processColumnsForHistogram(const QStringList& columnNames) {
-	ui->gbData->setTitle(i18n("Histogram Data"));
+	if (m_plotType == PlotType::Histogram)
+		ui->gbData->setTitle(i18n("Histogram Data"));
+	else
+		ui->gbData->setTitle(i18n("Box Plot Data"));
+
 	ui->line->hide();
 	ui->spacer->changeSize(0, 0);
 	ui->chkCreateDataCurve->hide();
@@ -342,13 +335,25 @@ void PlotDataDialog::processColumnsForHistogram(const QStringList& columnNames)
 		ui->cbYColumn->hide();
 		ui->rbCurvePlacement1->setChecked(true);
 		ui->gbCurvePlacement->hide();
-		ui->gbPlotPlacement->setTitle(i18n("Add Histogram to"));
+
+		if (m_plotType == PlotType::Histogram)
+			ui->gbPlotPlacement->setTitle(i18n("Add Histogram to"));
+		else
+			ui->gbPlotPlacement->setTitle(i18n("Add Box Plot to"));
+
 		ui->scrollAreaColumns->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 	} else {
-		ui->gbCurvePlacement->setTitle(i18n("Histogram Placement"));
-		ui->rbCurvePlacement1->setText(i18n("All histograms in one plot"));
-		ui->rbCurvePlacement2->setText(i18n("One plot per histogram"));
-		ui->gbPlotPlacement->setTitle(i18n("Add Histograms to"));
+		if (m_plotType == PlotType::Histogram) {
+			ui->gbCurvePlacement->setTitle(i18n("Histogram Placement"));
+			ui->rbCurvePlacement1->setText(i18n("All histograms in one plot"));
+			ui->rbCurvePlacement2->setText(i18n("One plot per histogram"));
+			ui->gbPlotPlacement->setTitle(i18n("Add Histograms to"));
+		} else {
+			ui->gbCurvePlacement->setTitle(i18n("Box Plot Placement"));
+			ui->rbCurvePlacement1->setText(i18n("All box plots in one plot"));
+			ui->rbCurvePlacement2->setText(i18n("One plot per box plot"));
+			ui->gbPlotPlacement->setTitle(i18n("Add Box Plots to"));
+		}
 		ui->scrollAreaColumns->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
 
 		//use the already available cbYColumn combo box
@@ -372,7 +377,6 @@ void PlotDataDialog::processColumnsForHistogram(const QStringList& columnNames)
 }
 
 void PlotDataDialog::plot() {
-	DEBUG("PlotDataDialog::plot()");
 	WAIT_CURSOR;
 	m_spreadsheet->project()->setSuppressAspectAddedSignal(true);
 	m_lastAddedCurve = nullptr;
@@ -392,32 +396,11 @@ void PlotDataDialog::plot() {
 
 		if (ui->rbCurvePlacement1->isChecked()) {
 			//all curves in one plot
-			CartesianPlot* plot = new CartesianPlot( i18n("Plot - %1", m_spreadsheet->name()) );
+			auto* plot = new CartesianPlot( i18n("Plot - %1", m_spreadsheet->name()) );
 			plot->setType(CartesianPlot::Type::FourAxes);
-
-			//set the axis titles before we add the plot to the worksheet
-			//set the x-axis names
-			const QString& xColumnName = ui->cbXColumn->currentText();
-			for (auto* axis : plot->children<Axis>()) {
-				if (axis->orientation() == Axis::Orientation::Horizontal) {
-					axis->title()->setText(xColumnName);
-					break;
-				}
-			}
-
-			//if we only have one single y-column to plot, we can set the title of the y-axes
-			if (m_columnComboBoxes.size() == 2) {
-				const QString& yColumnName = m_columnComboBoxes[1]->currentText();
-				for (auto* axis : plot->children<Axis>()) {
-					if (axis->orientation() == Axis::Orientation::Vertical) {
-						axis->title()->setText(yColumnName);
-						break;
-					}
-				}
-			}
-
 			worksheet->addChild(plot);
 			addCurvesToPlot(plot);
+			setAxesTitles(plot);
 		} else {
 			//one plot per curve
 			addCurvesToPlots(worksheet);
@@ -425,7 +408,7 @@ void PlotDataDialog::plot() {
 		worksheet->endMacro();
 	} else {
 		//add curves to a new plot(s) in a new worksheet
-		AbstractAspect* parent = m_spreadsheet->parentAspect();
+		auto* parent = m_spreadsheet->parentAspect();
 		if (parent->type() == AspectType::DatapickerCurve)
 			parent = parent->parentAspect()->parentAspect();
 		else if (parent->type() == AspectType::Workbook)
@@ -435,37 +418,16 @@ void PlotDataDialog::plot() {
 			parent = m_spreadsheet->project();
 #endif
 		parent->beginMacro( i18n("Plot data from %1", m_spreadsheet->name()) );
-		Worksheet* worksheet = new Worksheet(i18n("Worksheet - %1", m_spreadsheet->name()));
+		auto* worksheet = new Worksheet(i18n("Worksheet - %1", m_spreadsheet->name()));
 		parent->addChild(worksheet);
 
 		if (ui->rbCurvePlacement1->isChecked()) {
 			//all curves in one plot
-			CartesianPlot* plot = new CartesianPlot( i18n("Plot - %1", m_spreadsheet->name()) );
+			auto* plot = new CartesianPlot( i18n("Plot - %1", m_spreadsheet->name()) );
 			plot->setType(CartesianPlot::Type::FourAxes);
-
-			//set the axis titles before we add the plot to the worksheet
-			//set the x-axis names
-			const QString& xColumnName = ui->cbXColumn->currentText();
-			for (auto* axis : plot->children<Axis>()) {
-				if (axis->orientation() == Axis::Orientation::Horizontal) {
-					axis->title()->setText(xColumnName);
-					break;
-				}
-			}
-
-			//if we only have one single y-column to plot, we can set the title of the y-axes
-			if (m_columnComboBoxes.size() == 2) {
-				const QString& yColumnName = m_columnComboBoxes[1]->currentText();
-				for (auto* axis : plot->children<Axis>()) {
-					if (axis->orientation() == Axis::Orientation::Vertical) {
-						axis->title()->setText(yColumnName);
-						break;
-					}
-				}
-			}
-
 			worksheet->addChild(plot);
 			addCurvesToPlot(plot);
+			setAxesTitles(plot);
 		} else {
 			//one plot per curve
 			addCurvesToPlots(worksheet);
@@ -478,91 +440,26 @@ void PlotDataDialog::plot() {
 		//A more sophisticated and better logic would require further adjustments for properties like plot area
 		//paddings, the font size of axis ticks and title labels, etc. Also, this logic should be applied maybe if
 		//we add plots to an already created worksheet.
-
-		//adjust the sizes
-		const auto layout = worksheet->layout();
-		const auto plots = worksheet->children<CartesianPlot>();
-		const int count = plots.size();
-		const double minSize = 4.0;
-		switch (layout) {
-		case Worksheet::Layout::NoLayout:
-		case Worksheet::Layout::VerticalLayout: {
-			if (layout == Worksheet::Layout::NoLayout)
-				worksheet->setLayout(Worksheet::Layout::VerticalLayout);
-
-			const auto plot = plots.constFirst();
-			double height = Worksheet::convertFromSceneUnits(plot->rect().height(), Worksheet::Unit::Centimeter);
-			if (height < 4.) {
-				double newHeight = worksheet->layoutTopMargin() + worksheet->layoutBottomMargin()
-							+ (count - 1) * worksheet->layoutHorizontalSpacing()
-							+ count * Worksheet::convertToSceneUnits(minSize, Worksheet::Unit::Centimeter);
-				QRectF newRect = worksheet->pageRect();
-				newRect.setHeight(round(newHeight));
-				worksheet->setPageRect(newRect);
-			}
-			break;
-		}
-		case Worksheet::Layout::HorizontalLayout: {
-			const auto plot = plots.constFirst();
-			double width = Worksheet::convertFromSceneUnits(plot->rect().width(), Worksheet::Unit::Centimeter);
-			if (width < 4.) {
-				double newWidth = worksheet->layoutLeftMargin() + worksheet->layoutRightMargin()
-							+ (count - 1) * worksheet->layoutVerticalSpacing()
-							+ count * Worksheet::convertToSceneUnits(minSize, Worksheet::Unit::Centimeter);
-				QRectF newRect = worksheet->pageRect();
-				newRect.setWidth(round(newWidth));
-				worksheet->setPageRect(newRect);
-			}
-			break;
-		}
-		case Worksheet::Layout::GridLayout: {
-			const auto plot = plots.constFirst();
-			double width = Worksheet::convertFromSceneUnits(plot->rect().width(), Worksheet::Unit::Centimeter);
-			double height = Worksheet::convertFromSceneUnits(plot->rect().height(), Worksheet::Unit::Centimeter);
-			if (width < 4. || height < 4.) {
-				QRectF newRect = worksheet->pageRect();
-				if (height < 4.) {
-					double newHeight = worksheet->layoutTopMargin() + worksheet->layoutBottomMargin()
-									+ (count - 1) * worksheet->layoutHorizontalSpacing()
-									+ count * Worksheet::convertToSceneUnits(minSize, Worksheet::Unit::Centimeter);
-					newRect.setHeight(round(newHeight));
-				} else {
-					double newWidth = worksheet->layoutLeftMargin() + worksheet->layoutRightMargin()
-								+ (count - 1) * worksheet->layoutVerticalSpacing()
-								+ count * Worksheet::convertToSceneUnits(minSize, Worksheet::Unit::Centimeter);
-					newRect.setWidth(round(newWidth));
-				}
-				worksheet->setPageRect(newRect);
-			}
-			break;
-		}
-		}
+		adjustWorksheetSize(worksheet);
 
 		parent->endMacro();
 	}
 
-	//if more than two curves are produced, select the parent plot of the last added curve in the project explorer.
+	//if new curves are created via this dialog, select the parent plot of the last added curve in the project explorer.
 	//if a custom fit is being done, select the last created fit curve independent of the number of created curves.
 	m_spreadsheet->project()->setSuppressAspectAddedSignal(false);
 	if (m_lastAddedCurve) {
 		QString path;
 		if (!m_analysisMode) {
-			if (m_columns.size() > 2)
-				path = m_lastAddedCurve->parentAspect()->path();
-			else
-				path = m_lastAddedCurve->path();
+			path = m_lastAddedCurve->parentAspect()->path();
 		} else {
-			if (m_analysisAction == AnalysisAction::FitCustom)
+			if (m_analysisAction == XYAnalysisCurve::AnalysisAction::FitCustom)
 				path = m_lastAddedCurve->path();
-			else {
-				if (m_columns.size() > 2)
-					path = m_lastAddedCurve->parentAspect()->path();
-				else
-					path = m_lastAddedCurve->path();
-			}
+			else
+				path = m_lastAddedCurve->parentAspect()->path();
 		}
 
-		emit m_spreadsheet->project()->requestNavigateTo(path);
+		Q_EMIT m_spreadsheet->project()->requestNavigateTo(path);
 	}
 	RESET_CURSOR;
 }
@@ -604,9 +501,23 @@ void PlotDataDialog::addCurvesToPlot(CartesianPlot* plot) {
 		}
 		break;
 	}
+	case PlotType::BoxPlot: {
+		QVector<const AbstractColumn*> columns;
+		for (auto* comboBox : m_columnComboBoxes)
+			columns << columnFromName(comboBox->currentText());
+
+		QString name;
+		if (columns.size() > 1)
+			name = i18n("Box Plot");
+		else
+			name = columns.constFirst()->name();
+		addBoxPlot(name, columns, plot);
+		break;
+	}
 	}
 
-	plot->dataChanged();
+	// TODO: check if it really needs to update all ranges
+	plot->dataChanged(-1, -1);
 }
 
 /*!
@@ -626,25 +537,13 @@ void PlotDataDialog::addCurvesToPlots(Worksheet* worksheet) {
 			if (yColumn == xColumn)
 				continue;
 
-			CartesianPlot* plot = new CartesianPlot(i18n("Plot %1", name));
+			auto* plot = new CartesianPlot(i18n("Plot %1", name));
 			plot->setType(CartesianPlot::Type::FourAxes);
-
-			//set the axis names in the new plot
-			bool xSet = false;
-			bool ySet = false;
-			for (auto* axis : plot->children<Axis>()) {
-				if (axis->orientation() == Axis::Orientation::Horizontal && !xSet) {
-					axis->title()->setText(xColumnName);
-					xSet = true;
-				} else if (axis->orientation() == Axis::Orientation::Vertical && !ySet) {
-					axis->title()->setText(name);
-					ySet = true;
-				}
-			}
-
 			worksheet->addChild(plot);
 			addCurve(name, xColumn, yColumn, plot);
-			plot->scaleAuto();
+			plot->scaleAuto(-1, -1);
+			plot->retransform();
+			setAxesTitles(plot, name);
 		}
 		break;
 	}
@@ -653,22 +552,30 @@ void PlotDataDialog::addCurvesToPlots(Worksheet* worksheet) {
 			const QString& name = comboBox->currentText();
 			Column* column = columnFromName(name);
 
-			CartesianPlot* plot = new CartesianPlot(i18n("Plot %1", name));
+			auto* plot = new CartesianPlot(i18n("Plot %1", name));
 			plot->setType(CartesianPlot::Type::FourAxes);
-
-			//set the axis names in the new plot
-			bool xSet = false;
-			for (auto* axis : plot->children<Axis>()) {
-				if (axis->orientation() == Axis::Orientation::Horizontal && !xSet) {
-					axis->title()->setText(name);
-					xSet = true;
-				}
-			}
-
+			setAxesTitles(plot, name);
 			worksheet->addChild(plot);
 			addHistogram(name, column, plot);
-			plot->scaleAuto();
+			plot->scaleAuto(-1, -1);
+			plot->retransform();
 		}
+		break;
+	}
+	case PlotType::BoxPlot: {
+		for (auto* comboBox : m_columnComboBoxes) {
+			const QString& name = comboBox->currentText();
+			Column* column = columnFromName(name);
+
+			auto* plot = new CartesianPlot(i18n("Plot %1", name));
+			plot->setType(CartesianPlot::Type::FourAxes);
+			worksheet->addChild(plot);
+			addBoxPlot(name, QVector<const AbstractColumn*>{column}, plot);
+			plot->scaleAuto(-1, -1);
+			plot->retransform();
+			setAxesTitles(plot, name);
+		}
+		break;
 	}
 	}
 
@@ -680,7 +587,6 @@ void PlotDataDialog::addCurvesToPlots(Worksheet* worksheet) {
  * helper function that does the actual creation of the curve and adding it as child to the \c plot.
  */
 void PlotDataDialog::addCurve(const QString& name, Column* xColumn, Column* yColumn, CartesianPlot* plot) {
-	DEBUG("PlotDataDialog::addCurve()");
 	if (!m_analysisMode) {
 		auto* curve = new XYCurve(name);
 		curve->suppressRetransform(true);
@@ -704,37 +610,37 @@ void PlotDataDialog::addCurve(const QString& name, Column* xColumn, Column* yCol
 
 		XYAnalysisCurve* analysisCurve = nullptr;
 		switch (m_analysisAction) {
-			case AnalysisAction::DataReduction:
+			case XYAnalysisCurve::AnalysisAction::DataReduction:
 				analysisCurve = new XYDataReductionCurve(i18n("Reduction of '%1'", name));
 				break;
-			case AnalysisAction::Differentiation:
+			case XYAnalysisCurve::AnalysisAction::Differentiation:
 				analysisCurve = new XYDifferentiationCurve(i18n("Derivative of '%1'", name));
 				break;
-			case AnalysisAction::Integration:
+			case XYAnalysisCurve::AnalysisAction::Integration:
 				analysisCurve = new XYIntegrationCurve(i18n("Integral of '%1'", name));
 				break;
-			case AnalysisAction::Interpolation:
+			case XYAnalysisCurve::AnalysisAction::Interpolation:
 				analysisCurve = new XYInterpolationCurve(i18n("Interpolation of '%1'", name));
 				break;
-			case AnalysisAction::Smoothing:
+			case XYAnalysisCurve::AnalysisAction::Smoothing:
 				analysisCurve = new XYSmoothCurve(i18n("Smoothing of '%1'", name));
 				break;
-			case AnalysisAction::FitLinear:
-			case AnalysisAction::FitPower:
-			case AnalysisAction::FitExp1:
-			case AnalysisAction::FitExp2:
-			case AnalysisAction::FitInvExp:
-			case AnalysisAction::FitGauss:
-			case AnalysisAction::FitCauchyLorentz:
-			case AnalysisAction::FitTan:
-			case AnalysisAction::FitTanh:
-			case AnalysisAction::FitErrFunc:
-			case AnalysisAction::FitCustom:
+			case XYAnalysisCurve::AnalysisAction::FitLinear:
+			case XYAnalysisCurve::AnalysisAction::FitPower:
+			case XYAnalysisCurve::AnalysisAction::FitExp1:
+			case XYAnalysisCurve::AnalysisAction::FitExp2:
+			case XYAnalysisCurve::AnalysisAction::FitInvExp:
+			case XYAnalysisCurve::AnalysisAction::FitGauss:
+			case XYAnalysisCurve::AnalysisAction::FitCauchyLorentz:
+			case XYAnalysisCurve::AnalysisAction::FitTan:
+			case XYAnalysisCurve::AnalysisAction::FitTanh:
+			case XYAnalysisCurve::AnalysisAction::FitErrFunc:
+			case XYAnalysisCurve::AnalysisAction::FitCustom:
 				analysisCurve = new XYFitCurve(i18n("Fit to '%1'", name));
 				static_cast<XYFitCurve*>(analysisCurve)->initFitData(m_analysisAction);
 				static_cast<XYFitCurve*>(analysisCurve)->initStartValues(curve);
 				break;
-			case AnalysisAction::FourierFilter:
+			case XYAnalysisCurve::AnalysisAction::FourierFilter:
 				analysisCurve = new XYFourierFilterCurve(i18n("Fourier Filter of '%1'", name));
 				break;
 		}
@@ -743,7 +649,7 @@ void PlotDataDialog::addCurve(const QString& name, Column* xColumn, Column* yCol
 			analysisCurve->suppressRetransform(true);
 			analysisCurve->setXDataColumn(xColumn);
 			analysisCurve->setYDataColumn(yColumn);
-			if (m_analysisAction != AnalysisAction::FitCustom) //no custom fit-model set yet, no need to recalculate
+			if (m_analysisAction != XYAnalysisCurve::AnalysisAction::FitCustom) //no custom fit-model set yet, no need to recalculate
 				analysisCurve->recalculate();
 			analysisCurve->suppressRetransform(false);
 			plot->addChild(analysisCurve);
@@ -754,13 +660,192 @@ void PlotDataDialog::addCurve(const QString& name, Column* xColumn, Column* yCol
 
 void PlotDataDialog::addHistogram(const QString& name, Column* column, CartesianPlot* plot) {
 	auto* hist = new Histogram(name);
-	plot->addChild(hist);
 // 	hist->suppressRetransform(true);
 	hist->setDataColumn(column);
 // 	hist->suppressRetransform(false);
+	plot->addChild(hist);
 	m_lastAddedCurve = hist;
 }
 
+void PlotDataDialog::addBoxPlot(const QString& name, const QVector<const AbstractColumn*>& columns, CartesianPlot* plot) {
+	QApplication::processEvents(QEventLoop::AllEvents, 100);
+	auto* boxPlot = new BoxPlot(name);
+	boxPlot->setDataColumns(columns);
+	plot->addChild(boxPlot);
+	m_lastAddedCurve = boxPlot;
+}
+
+void PlotDataDialog::adjustWorksheetSize(Worksheet* worksheet) const {
+	//adjust the sizes
+	const auto layout = worksheet->layout();
+	const auto plots = worksheet->children<CartesianPlot>();
+	const int count = plots.size();
+	const double minSize = 4.0;
+	switch (layout) {
+	case Worksheet::Layout::NoLayout:
+	case Worksheet::Layout::VerticalLayout: {
+		if (layout == Worksheet::Layout::NoLayout)
+			worksheet->setLayout(Worksheet::Layout::VerticalLayout);
+
+		const auto plot = plots.constFirst();
+		double height = Worksheet::convertFromSceneUnits(plot->rect().height(), Worksheet::Unit::Centimeter);
+		if (height < 4.) {
+			double newHeight = worksheet->layoutTopMargin() + worksheet->layoutBottomMargin()
+						+ (count - 1) * worksheet->layoutHorizontalSpacing()
+						+ count * Worksheet::convertToSceneUnits(minSize, Worksheet::Unit::Centimeter);
+			QRectF newRect = worksheet->pageRect();
+			newRect.setHeight(round(newHeight));
+			worksheet->setPageRect(newRect);
+		}
+		break;
+	}
+	case Worksheet::Layout::HorizontalLayout: {
+		const auto plot = plots.constFirst();
+		double width = Worksheet::convertFromSceneUnits(plot->rect().width(), Worksheet::Unit::Centimeter);
+		if (width < 4.) {
+			double newWidth = worksheet->layoutLeftMargin() + worksheet->layoutRightMargin()
+						+ (count - 1) * worksheet->layoutVerticalSpacing()
+						+ count * Worksheet::convertToSceneUnits(minSize, Worksheet::Unit::Centimeter);
+			QRectF newRect = worksheet->pageRect();
+			newRect.setWidth(round(newWidth));
+			worksheet->setPageRect(newRect);
+		}
+		break;
+	}
+	case Worksheet::Layout::GridLayout: {
+		const auto plot = plots.constFirst();
+		double width = Worksheet::convertFromSceneUnits(plot->rect().width(), Worksheet::Unit::Centimeter);
+		double height = Worksheet::convertFromSceneUnits(plot->rect().height(), Worksheet::Unit::Centimeter);
+		if (width < 4. || height < 4.) {
+			QRectF newRect = worksheet->pageRect();
+			if (height < 4.) {
+				double newHeight = worksheet->layoutTopMargin() + worksheet->layoutBottomMargin()
+								+ (count - 1) * worksheet->layoutHorizontalSpacing()
+								+ count * Worksheet::convertToSceneUnits(minSize, Worksheet::Unit::Centimeter);
+				newRect.setHeight(round(newHeight));
+			} else {
+				double newWidth = worksheet->layoutLeftMargin() + worksheet->layoutRightMargin()
+							+ (count - 1) * worksheet->layoutVerticalSpacing()
+							+ count * Worksheet::convertToSceneUnits(minSize, Worksheet::Unit::Centimeter);
+				newRect.setWidth(round(newWidth));
+			}
+			worksheet->setPageRect(newRect);
+		}
+		break;
+	}
+	}
+}
+
+void PlotDataDialog::setAxesTitles(CartesianPlot* plot, const QString& name) const {
+	DEBUG(Q_FUNC_INFO)
+	const auto& axes = plot->children<Axis>();
+	switch (m_plotType) {
+	case PlotType::XYCurve: {
+		//x-axis title
+		const QString& xColumnName = ui->cbXColumn->currentText();
+		//DEBUG(Q_FUNC_INFO << ", x column name = " << STDSTRING(xColumnName))
+
+		for (auto* axis : axes) {
+			if (axis->orientation() == Axis::Orientation::Horizontal) {
+				axis->title()->setText(xColumnName);
+				break;
+			}
+		}
+
+		//y-axis title
+		for (auto* axis : axes) {
+			if (axis->orientation() == Axis::Orientation::Vertical) {
+				if (!name.isEmpty()) {
+					//multiple columns are plotted with "one curve per plot",
+					//the function is called with the column name.
+					//use it for the x-axis title
+					axis->title()->setText(name);
+				} else if (m_columnComboBoxes.size() == 2) {
+					//if we only have one single y-column to plot, we can set the title of the y-axes
+					const QString& yColumnName = m_columnComboBoxes[1]->currentText();
+					axis->title()->setText(yColumnName);
+				}
+
+				break;
+			}
+		}
+		break;
+	}
+	case PlotType::Histogram: {
+		//x-axis title
+		for (auto* axis : axes) {
+			if (axis->orientation() == Axis::Orientation::Horizontal) {
+				if (!name.isEmpty()) {
+					//multiple columns are plotted with "one curve per plot",
+					//the function is called with the column name.
+					//use it for the x-axis title
+					axis->title()->setText(name);
+				} else if (m_columnComboBoxes.size() == 1) {
+					const QString& yColumnName = m_columnComboBoxes.constFirst()->currentText();
+					axis->title()->setText(yColumnName);
+				}
+
+				break;
+			}
+		}
+
+
+		//y-axis title
+		for (auto* axis : axes) {
+			if (axis->orientation() == Axis::Orientation::Vertical) {
+				axis->title()->setText(i18n("Frequency"));
+				break;
+			}
+		}
+		break;
+	}
+	case PlotType::BoxPlot: {
+		auto* boxPlot = static_cast<BoxPlot*>(m_lastAddedCurve);
+		auto orientation = boxPlot->orientation();
+
+		//number of box plots per plot:
+		int count = 1; //1 if we create a separate plot for every box plot
+		if (ui->rbCurvePlacement1->isChecked())
+			count = m_columnComboBoxes.count(); //all box plots in one single plot
+
+		//set the range of the plot and the number of ticks manuall.
+		//the n-th box plot is positioned at x=n and and has the width=0.5 in logical coordinatates.
+		//manually set the range to (0.5, n+0.5) and ajdust the number of ticks starting at 0.5
+		//to make sure we have every tick precisely under the middle of the box plot
+		Range<double> range(0.5, count + 0.5);
+		if (orientation == WorksheetElement::Orientation::Vertical)
+			plot->setXRange(range);
+		else
+			plot->setYRange(range);
+
+		for (auto* axis : axes) {
+			if (axis->orientation() != orientation) {
+				axis->setMajorTicksNumber(count + 1);
+				axis->setMajorTickStartOffset(0.5);
+				axis->setMinorTicksNumber(0);
+			}
+			axis->title()->setText(QString()); //no title
+		}
+
+		//y-axis title
+		for (auto* axis : axes) {
+			if (axis->orientation() == Axis::Orientation::Vertical) {
+				if (!name.isEmpty()) {
+					//multiple columns are plotted with "one curve per plot",
+					//the function is called with the column name.
+					//use it for the x-axis title
+					axis->title()->setText(name);
+				} else if (count == 1) {
+					const QString& yColumnName = m_columnComboBoxes.constFirst()->currentText();
+					axis->title()->setText(yColumnName);
+				}
+				break;
+			}
+		}
+	}
+	}
+}
+
 //################################################################
 //########################## Slots ###############################
 //################################################################
