@@ -1,45 +1,25 @@
-/***************************************************************************
+/*
     File                 : DatapickerImageWidget.cpp
     Project              : LabPlot
     Description          : widget for datapicker properties
     --------------------------------------------------------------------
-    Copyright            : (C) 2015-2016 by Ankit Wagadre (wagadre.ankit@gmail.com)
-    Copyright            : (C) 2015-2019 by Alexander Semke (alexander.semke@web.de)
+    SPDX-FileCopyrightText: 2015-2016 Ankit Wagadre <wagadre.ankit@gmail.com>
+    SPDX-FileCopyrightText: 2015-2021 Alexander Semke <alexander.semke@web.de>
 
- ***************************************************************************/
-/***************************************************************************
- *                                                                         *
- *  This program is free software; you can redistribute it and/or modify   *
- *  it under the terms of the GNU General Public License as published by   *
- *  the Free Software Foundation; either version 2 of the License, or      *
- *  (at your option) any later version.                                    *
- *                                                                         *
- *  This program is distributed in the hope that it will be useful,        *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
- *  GNU General Public License for more details.                           *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the Free Software           *
- *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
- *   Boston, MA  02110-1301  USA                                           *
- *                                                                         *
- ***************************************************************************/
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "DatapickerImageWidget.h"
 #include "backend/datapicker/DatapickerPoint.h"
 #include "commonfrontend/widgets/qxtspanslider.h"
-#include "kdefrontend/GuiTools.h"
-#include "backend/worksheet/Worksheet.h"
-#include "backend/worksheet/plots/cartesian/Symbol.h"
 #include "backend/datapicker/ImageEditor.h"
+#include "kdefrontend/GuiTools.h"
+#include "kdefrontend/widgets/SymbolWidget.h"
 
 #include <QCompleter>
 #include <QDir>
-#include <QFileDialog>
 #include <QDirModel>
 #include <QGraphicsScene>
-#include <QImageReader>
 #include <QPainter>
 
 #include <KConfigGroup>
@@ -75,7 +55,7 @@ HistogramView::HistogramView(QWidget* parent, int range) : QGraphicsView(parent)
 void HistogramView::setScalePixmap(const QString& file) {
 	// scene rect is 1000*100 where upper 1000*80 is for histogram graph
 	// and lower 1000*20 is for histogram scale
-	QGraphicsPixmapItem* pixmap = new QGraphicsPixmapItem(QPixmap(file).scaled( 1000, 20, Qt::IgnoreAspectRatio), nullptr);
+	auto* pixmap = new QGraphicsPixmapItem(QPixmap(file).scaled( 1000, 20, Qt::IgnoreAspectRatio), nullptr);
 	pixmap->setZValue(-1);
 	pixmap->setPos(0, 90);
 	m_scene->addItem(pixmap);
@@ -126,38 +106,46 @@ void HistogramView::drawBackground(QPainter* painter, const QRectF& rect) {
 DatapickerImageWidget::DatapickerImageWidget(QWidget* parent) : BaseDock(parent), m_image(nullptr) {
 	ui.setupUi(this);
 	m_leName = ui.leName;
-	m_leComment = ui.leComment;
+	m_teComment = ui.teComment;
 
+	//"General"-tab
 	ui.leFileName->setClearButtonEnabled(true);
 	ui.bOpen->setIcon( QIcon::fromTheme("document-open") );
 	ui.leFileName->setCompleter(new QCompleter(new QDirModel, this));
 
+	//"Symbol"-tab
+	symbolWidget = new SymbolWidget(ui.tSymbol);
+	auto* gridLayout = dynamic_cast<QGridLayout*>(ui.tSymbol->layout());
+	if (gridLayout)
+		gridLayout->addWidget(symbolWidget, 0, 0, 1, 1);
+
+	//"Edit Image"-tab
 	auto* editTabLayout = static_cast<QGridLayout*>(ui.tEdit->layout());
 	editTabLayout->setContentsMargins(2,2,2,2);
 	editTabLayout->setHorizontalSpacing(2);
 	editTabLayout->setVerticalSpacing(4);
 
-	ssHue = new QxtSpanSlider(Qt::Horizontal, ui.tEdit);
+	ssHue = new SpanSlider(Qt::Horizontal, ui.tEdit);
 	ssHue->setToolTip(i18n("Select the range for the hue.\nEverything outside of this range will be set to white."));
 	ssHue->setRange(0, 360);
 	editTabLayout->addWidget(ssHue, 3, 2);
 
-	ssSaturation = new QxtSpanSlider(Qt::Horizontal, ui.tEdit);
+	ssSaturation = new SpanSlider(Qt::Horizontal, ui.tEdit);
 	ssSaturation->setToolTip(i18n("Select the range for the saturation.\nEverything outside of this range will be set to white."));
 	ssSaturation->setRange(0,100);
 	editTabLayout->addWidget(ssSaturation, 5, 2);
 
-	ssValue = new QxtSpanSlider(Qt::Horizontal, ui.tEdit);
+	ssValue = new SpanSlider(Qt::Horizontal, ui.tEdit);
 	ssValue->setToolTip(i18n("Select the range for the value, the degree of lightness of the color.\nEverything outside of this range will be set to white."));
 	ssValue->setRange(0,100);
 	editTabLayout->addWidget(ssValue, 7, 2);
 
-	ssIntensity = new QxtSpanSlider(Qt::Horizontal, ui.tEdit);
+	ssIntensity = new SpanSlider(Qt::Horizontal, ui.tEdit);
 	ssIntensity->setToolTip(i18n("Select the range for the intensity.\nEverything outside of this range will be set to white."));
 	ssIntensity->setRange(0, 100);
 	editTabLayout->addWidget(ssIntensity, 9, 2);
 
-	ssForeground = new QxtSpanSlider(Qt::Horizontal, ui.tEdit);
+	ssForeground = new SpanSlider(Qt::Horizontal, ui.tEdit);
 	ssForeground->setToolTip(i18n("Select the range for the colors that are not part of the background color.\nEverything outside of this range will be set to white."));
 	ssForeground->setRange(0, 100);
 	editTabLayout->addWidget(ssForeground, 11, 2);
@@ -211,12 +199,12 @@ DatapickerImageWidget::DatapickerImageWidget(QWidget* parent) : BaseDock(parent)
 	editTabLayout->addWidget(gvForeground, 10, 2);
 	gvForeground->setScalePixmap(valueFile);
 
-	updateLocale();
+	DatapickerImageWidget::updateLocale();
 
 	//SLOTS
 	//general
 	connect(ui.leName, &QLineEdit::textChanged, this, &DatapickerImageWidget::nameChanged);
-	connect(ui.leComment, &QLineEdit::textChanged, this, &DatapickerImageWidget::commentChanged);
+	connect(ui.teComment, &QTextEdit::textChanged, this, &DatapickerImageWidget::commentChanged);
 	connect(ui.bOpen, &QPushButton::clicked, this, &DatapickerImageWidget::selectFile);
 	connect(ui.leFileName, &QLineEdit::returnPressed, this, &DatapickerImageWidget::fileNameChanged);
 	connect(ui.leFileName, &QLineEdit::textChanged, this, &DatapickerImageWidget::fileNameChanged);
@@ -226,16 +214,16 @@ DatapickerImageWidget::DatapickerImageWidget(QWidget* parent) : BaseDock(parent)
 			this, &DatapickerImageWidget::plotImageTypeChanged);
 	connect(ui.sbRotation, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
 			this, &DatapickerImageWidget::rotationChanged);
-	connect(ssIntensity, &QxtSpanSlider::spanChanged, this, &DatapickerImageWidget::intensitySpanChanged);
-	connect(ssIntensity, &QxtSpanSlider::spanChanged, gvIntensity, &HistogramView::setSpan);
-	connect(ssForeground, &QxtSpanSlider::spanChanged, this, &DatapickerImageWidget::foregroundSpanChanged);
-	connect(ssForeground, &QxtSpanSlider::spanChanged, gvForeground, &HistogramView::setSpan );
-	connect(ssHue, &QxtSpanSlider::spanChanged, this, &DatapickerImageWidget::hueSpanChanged);
-	connect(ssHue, &QxtSpanSlider::spanChanged, gvHue, &HistogramView::setSpan );
-	connect(ssSaturation, &QxtSpanSlider::spanChanged, this, &DatapickerImageWidget::saturationSpanChanged);
-	connect(ssSaturation, &QxtSpanSlider::spanChanged, gvSaturation, &HistogramView::setSpan );
-	connect(ssValue, &QxtSpanSlider::spanChanged, this, &DatapickerImageWidget::valueSpanChanged);
-	connect(ssValue, &QxtSpanSlider::spanChanged, gvValue, &HistogramView::setSpan );
+	connect(ssIntensity, &SpanSlider::spanChanged, this, &DatapickerImageWidget::intensitySpanChanged);
+	connect(ssIntensity, &SpanSlider::spanChanged, gvIntensity, &HistogramView::setSpan);
+	connect(ssForeground, &SpanSlider::spanChanged, this, &DatapickerImageWidget::foregroundSpanChanged);
+	connect(ssForeground, &SpanSlider::spanChanged, gvForeground, &HistogramView::setSpan );
+	connect(ssHue, &SpanSlider::spanChanged, this, &DatapickerImageWidget::hueSpanChanged);
+	connect(ssHue, &SpanSlider::spanChanged, gvHue, &HistogramView::setSpan );
+	connect(ssSaturation, &SpanSlider::spanChanged, this, &DatapickerImageWidget::saturationSpanChanged);
+	connect(ssSaturation, &SpanSlider::spanChanged, gvSaturation, &HistogramView::setSpan );
+	connect(ssValue, &SpanSlider::spanChanged, this, &DatapickerImageWidget::valueSpanChanged);
+	connect(ssValue, &SpanSlider::spanChanged, gvValue, &HistogramView::setSpan );
 	connect(ui.sbMinSegmentLength, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
 			this, &DatapickerImageWidget::minSegmentLengthChanged);
 	connect(ui.sbPointSeparation, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
@@ -265,58 +253,7 @@ DatapickerImageWidget::DatapickerImageWidget(QWidget* parent) : BaseDock(parent)
 	connect(ui.sbPositionZ3, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
 			this, &DatapickerImageWidget::logicalPositionChanged);
 
-	//SYMBOL
-	connect(ui.cbSymbolStyle, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
-			this, &DatapickerImageWidget::pointsStyleChanged);
-	connect(ui.sbSymbolSize, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
-			this, &DatapickerImageWidget::pointsSizeChanged);
-	connect(ui.sbSymbolRotation,static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
-			this, &DatapickerImageWidget::pointsRotationChanged);
-	connect(ui.sbSymbolOpacity, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
-			this, &DatapickerImageWidget::pointsOpacityChanged);
-
-	//Filling
-	connect(ui.cbSymbolFillingStyle, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
-			this, &DatapickerImageWidget::pointsFillingStyleChanged);
-	connect(ui.kcbSymbolFillingColor, &KColorButton::changed, this, &DatapickerImageWidget::pointsFillingColorChanged);
-
-	//border
-	connect(ui.cbSymbolBorderStyle, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
-			this, &DatapickerImageWidget::pointsBorderStyleChanged);
-	connect(ui.kcbSymbolBorderColor, &KColorButton::changed, this, &DatapickerImageWidget::pointsBorderColorChanged);
-	connect(ui.sbSymbolBorderWidth, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
-			this, &DatapickerImageWidget::pointsBorderWidthChanged);
-
 	connect(ui.chbSymbolVisible, &QCheckBox::clicked, this, &DatapickerImageWidget::pointsVisibilityChanged);
-
-	init();
-}
-
-void DatapickerImageWidget::init() {
-	m_initializing = true;
-	GuiTools::updatePenStyles(ui.cbSymbolBorderStyle, Qt::black);
-
-	QPainter pa;
-	int iconSize = 20;
-	QPixmap pm(iconSize, iconSize);
-	QPen pen(Qt::SolidPattern, 0);
-	ui.cbSymbolStyle->setIconSize(QSize(iconSize, iconSize));
-	QTransform trafo;
-	trafo.scale(15, 15);
-	//TODO: constant?
-	for (int i = 1; i < 19; ++i) {
-		auto style = (Symbol::Style)i;
-		pm.fill(Qt::transparent);
-		pa.begin(&pm);
-		pa.setPen( pen );
-		pa.setRenderHint(QPainter::Antialiasing);
-		pa.translate(iconSize/2,iconSize/2);
-		pa.drawPath(trafo.map(Symbol::pathFromStyle(style)));
-		pa.end();
-		ui.cbSymbolStyle->addItem(QIcon(pm), Symbol::nameFromStyle(style));
-	}
-	GuiTools::updateBrushStyles(ui.cbSymbolFillingStyle, Qt::black);
-	m_initializing = false;
 }
 
 void DatapickerImageWidget::setImages(QList<DatapickerImage*> list) {
@@ -329,27 +266,27 @@ void DatapickerImageWidget::setImages(QList<DatapickerImage*> list) {
 		ui.lName->setEnabled(true);
 		ui.leName->setEnabled(true);
 		ui.lComment->setEnabled(true);
-		ui.leComment->setEnabled(true);
+		ui.teComment->setEnabled(true);
 		ui.leName->setText(m_image->parentAspect()->name());
-		ui.leComment->setText(m_image->parentAspect()->comment());
+		ui.teComment->setText(m_image->parentAspect()->comment());
 	} else {
 		ui.lName->setEnabled(false);
 		ui.leName->setEnabled(false);
 		ui.lComment->setEnabled(false);
-		ui.leComment->setEnabled(false);
+		ui.teComment->setEnabled(false);
 		ui.leName->setText(QString());
-		ui.leComment->setText(QString());
+		ui.teComment->setText(QString());
 	}
 
 	this->load();
-	initConnections();
-	handleWidgetActions();
-	updateSymbolWidgets();
-	m_initializing = false;
-}
 
-void DatapickerImageWidget::initConnections() {
-	connect(m_image->parentAspect(), &AbstractAspect::aspectDescriptionChanged, this, &DatapickerImageWidget::imageDescriptionChanged);
+	QList<Symbol*> symbols;
+	for (auto* image : m_imagesList)
+		symbols << image->symbol();
+
+	symbolWidget->setSymbols(symbols);
+
+	connect(m_image->parentAspect(), &AbstractAspect::aspectDescriptionChanged, this, &DatapickerImageWidget::aspectDescriptionChanged);
 	connect(m_image, &DatapickerImage::fileNameChanged, this, &DatapickerImageWidget::imageFileNameChanged);
 	connect(m_image, &DatapickerImage::rotationAngleChanged, this, &DatapickerImageWidget::imageRotationAngleChanged);
 	connect(m_image, &AbstractAspect::aspectRemoved, this, &DatapickerImageWidget::updateSymbolWidgets);
@@ -357,13 +294,11 @@ void DatapickerImageWidget::initConnections() {
 	connect(m_image, &DatapickerImage::axisPointsChanged, this, &DatapickerImageWidget::imageAxisPointsChanged);
 	connect(m_image, &DatapickerImage::settingsChanged, this, &DatapickerImageWidget::imageEditorSettingsChanged);
 	connect(m_image, &DatapickerImage::minSegmentLengthChanged, this, &DatapickerImageWidget::imageMinSegmentLengthChanged);
-	connect(m_image, &DatapickerImage::pointStyleChanged, this, &DatapickerImageWidget::symbolStyleChanged);
-	connect(m_image, &DatapickerImage::pointSizeChanged, this, &DatapickerImageWidget::symbolSizeChanged);
-	connect(m_image, &DatapickerImage::pointRotationAngleChanged, this, &DatapickerImageWidget::symbolRotationAngleChanged);
-	connect(m_image, &DatapickerImage::pointOpacityChanged, this, &DatapickerImageWidget::symbolOpacityChanged);
-	connect(m_image, &DatapickerImage::pointBrushChanged, this, &DatapickerImageWidget::symbolBrushChanged);
-	connect(m_image, &DatapickerImage::pointPenChanged, this, &DatapickerImageWidget::symbolPenChanged);
 	connect(m_image, &DatapickerImage::pointVisibilityChanged, this, &DatapickerImageWidget::symbolVisibleChanged);
+
+	handleWidgetActions();
+	updateSymbolWidgets();
+	m_initializing = false;
 }
 
 void DatapickerImageWidget::handleWidgetActions() {
@@ -400,8 +335,6 @@ void DatapickerImageWidget::updateLocale() {
 	ui.sbPositionY1->setLocale(numberLocale);
 	ui.sbPositionY2->setLocale(numberLocale);
 	ui.sbPositionY3->setLocale(numberLocale);
-	ui.sbSymbolSize->setLocale(numberLocale);
-	ui.sbSymbolBorderWidth->setLocale(numberLocale);
 }
 
 //**********************************************************
@@ -409,31 +342,11 @@ void DatapickerImageWidget::updateLocale() {
 //**********************************************************
 //"General"-tab
 void DatapickerImageWidget::selectFile() {
-	KConfigGroup conf(KSharedConfig::openConfig(), "DatapickerImageWidget");
-	QString dir = conf.readEntry("LastImageDir", "");
-	QString formats;
-	for (const QByteArray& format : QImageReader::supportedImageFormats()) {
-		QString f = "*." + QString(format.constData());
-		if (f == QLatin1String("*.svg"))
-			continue;
-		formats.isEmpty() ? formats += f : formats += " " + f;
-	}
-	QString path = QFileDialog::getOpenFileName(this, i18n("Select the image file"), dir, i18n("Images (%1)", formats));
+	const QString& path = GuiTools::openImageFile(QLatin1String("DatapickerImageWidget"));
 	if (path.isEmpty())
-		return; //cancel was clicked in the file-dialog
+		return;
 
-	int pos = path.lastIndexOf(QLatin1String("/"));
-	if (pos != -1) {
-		QString newDir = path.left(pos);
-		if (newDir != dir)
-			conf.writeEntry("LastImageDir", newDir);
-	}
-
-	ui.leFileName->setText( path );
-	handleWidgetActions();
-
-	for (auto* image : m_imagesList)
-		image->setFileName(path);
+	ui.leFileName->setText(path);
 }
 
 void DatapickerImageWidget::fileNameChanged() {
@@ -442,41 +355,45 @@ void DatapickerImageWidget::fileNameChanged() {
 
 	handleWidgetActions();
 
-	QString fileName = ui.leFileName->text();
-	if (!fileName.isEmpty() && !QFile::exists(fileName))
-		ui.leFileName->setStyleSheet("QLineEdit{background:red;}");
-	else
-		ui.leFileName->setStyleSheet(QString());
+	const QString& fileName = ui.leFileName->text();
+	bool invalid = (!fileName.isEmpty() && !QFile::exists(fileName));
+	GuiTools::highlight(ui.leFileName, invalid);
 
 	for (auto* image : m_imagesList)
 		image->setFileName(fileName);
 }
 
 void DatapickerImageWidget::graphTypeChanged() {
+	auto points = m_image->axisPoints();
+	points.type = DatapickerImage::GraphType(ui.cbGraphType->currentIndex());
+
+	const bool ternary = (points.type == DatapickerImage::GraphType::Ternary);
+	ui.lTernaryScale->setVisible(ternary);
+	ui.sbTernaryScale->setVisible(ternary);
+	ui.lPositionZ1->setVisible(ternary);
+	ui.lPositionZ2->setVisible(ternary);
+	ui.lPositionZ3->setVisible(ternary);
+	ui.sbPositionZ1->setVisible(ternary);
+	ui.sbPositionZ2->setVisible(ternary);
+	ui.sbPositionZ3->setVisible(ternary);
+
 	if (m_initializing)
 		return;
 
-	DatapickerImage::ReferencePoints points = m_image->axisPoints();
-	points.type = DatapickerImage::GraphType(ui.cbGraphType->currentIndex());
-
-	if (points.type != DatapickerImage::GraphType::Ternary) {
-		ui.lTernaryScale->setHidden(true);
-		ui.sbTernaryScale->setHidden(true);
-		ui.lPositionZ1->setHidden(true);
-		ui.lPositionZ2->setHidden(true);
-		ui.lPositionZ3->setHidden(true);
-		ui.sbPositionZ1->setHidden(true);
-		ui.sbPositionZ2->setHidden(true);
-		ui.sbPositionZ3->setHidden(true);
-	} else {
-		ui.lTernaryScale->setHidden(false);
-		ui.sbTernaryScale->setHidden(false);
-		ui.lPositionZ1->setHidden(false);
-		ui.lPositionZ2->setHidden(false);
-		ui.lPositionZ3->setHidden(false);
-		ui.sbPositionZ1->setHidden(false);
-		ui.sbPositionZ2->setHidden(false);
-		ui.sbPositionZ3->setHidden(false);
+	if (ui.cbGraphType->currentIndex() == 3) { //"Logarithmic (ln(x), y)"
+		if (points.logicalPos[0].x() == 0.0f)
+			points.logicalPos[0].setX(0.01f);
+		if (points.logicalPos[1].x() == 0.0f)
+			points.logicalPos[1].setX(0.01f);
+		if (points.logicalPos[2].x() == 0.0f)
+			points.logicalPos[2].setX(0.01f);
+	} else if (ui.cbGraphType->currentIndex() == 4) { //"Logarithmic (x, ln(y))"
+		if (points.logicalPos[0].y() == 0.0f)
+			points.logicalPos[0].setY(0.01f);
+		if (points.logicalPos[1].y() == 0.0f)
+			points.logicalPos[1].setY(0.01f);
+		if (points.logicalPos[2].y() == 0.0f)
+			points.logicalPos[2].setY(0.01f);
 	}
 
 	for (auto* image : m_imagesList)
@@ -498,7 +415,7 @@ void DatapickerImageWidget::logicalPositionChanged() {
 	if (m_initializing)
 		return;
 
-	DatapickerImage::ReferencePoints points = m_image->axisPoints();
+	auto points = m_image->axisPoints();
 	points.logicalPos[0].setX(ui.sbPositionX1->value());
 	points.logicalPos[0].setY(ui.sbPositionY1->value());
 	points.logicalPos[1].setX(ui.sbPositionX2->value());
@@ -514,135 +431,6 @@ void DatapickerImageWidget::logicalPositionChanged() {
 		image->setAxisPoints(points);
 }
 
-void DatapickerImageWidget::pointsStyleChanged(int index) {
-	auto style = Symbol::Style(index + 1);
-	//enable/disable the  filling options in the GUI depending on the currently selected points.
-	if (style != Symbol::Style::Line && style != Symbol::Style::Cross) {
-		ui.cbSymbolFillingStyle->setEnabled(true);
-		bool noBrush = (Qt::BrushStyle(ui.cbSymbolFillingStyle->currentIndex()) == Qt::NoBrush);
-		ui.kcbSymbolFillingColor->setEnabled(!noBrush);
-	} else {
-		ui.kcbSymbolFillingColor->setEnabled(false);
-		ui.cbSymbolFillingStyle->setEnabled(false);
-	}
-
-	bool noLine = (Qt::PenStyle(ui.cbSymbolBorderStyle->currentIndex()) == Qt::NoPen);
-	ui.kcbSymbolBorderColor->setEnabled(!noLine);
-	ui.sbSymbolBorderWidth->setEnabled(!noLine);
-
-	if (m_initializing)
-		return;
-
-	for (auto* image : m_imagesList)
-		image->setPointStyle(style);
-}
-
-void DatapickerImageWidget::pointsSizeChanged(double value) {
-	if (m_initializing)
-		return;
-
-	for (auto* image : m_imagesList)
-		image->setPointSize( Worksheet::convertToSceneUnits(value, Worksheet::Unit::Point) );
-}
-
-void DatapickerImageWidget::pointsRotationChanged(int value) {
-	if (m_initializing)
-		return;
-
-	for (auto* image : m_imagesList)
-		image->setPointRotationAngle(value);
-}
-
-void DatapickerImageWidget::pointsOpacityChanged(int value) {
-	if (m_initializing)
-		return;
-
-	qreal opacity = (float)value/100.;
-	for (auto* image : m_imagesList)
-		image->setPointOpacity(opacity);
-}
-
-void DatapickerImageWidget::pointsFillingStyleChanged(int index) {
-	auto brushStyle = Qt::BrushStyle(index);
-	ui.kcbSymbolFillingColor->setEnabled(!(brushStyle == Qt::NoBrush));
-
-	if (m_initializing)
-		return;
-
-	QBrush brush;
-	for (auto* image : m_imagesList) {
-		brush = image->pointBrush();
-		brush.setStyle(brushStyle);
-		image->setPointBrush(brush);
-	}
-}
-
-void DatapickerImageWidget::pointsFillingColorChanged(const QColor& color) {
-	if (m_initializing)
-		return;
-
-	QBrush brush;
-	for (auto* image : m_imagesList) {
-		brush = image->pointBrush();
-		brush.setColor(color);
-		image->setPointBrush(brush);
-	}
-
-	m_initializing = true;
-	GuiTools::updateBrushStyles(ui.cbSymbolFillingStyle, color );
-	m_initializing = false;
-}
-
-void DatapickerImageWidget::pointsBorderStyleChanged(int index) {
-	auto penStyle = Qt::PenStyle(index);
-
-	if ( penStyle == Qt::NoPen ) {
-		ui.kcbSymbolBorderColor->setEnabled(false);
-		ui.sbSymbolBorderWidth->setEnabled(false);
-	} else {
-		ui.kcbSymbolBorderColor->setEnabled(true);
-		ui.sbSymbolBorderWidth->setEnabled(true);
-	}
-
-	if (m_initializing)
-		return;
-
-	QPen pen;
-	for (auto* image : m_imagesList) {
-		pen = image->pointPen();
-		pen.setStyle(penStyle);
-		image->setPointPen(pen);
-	}
-}
-
-void DatapickerImageWidget::pointsBorderColorChanged(const QColor& color) {
-	if (m_initializing)
-		return;
-
-	QPen pen;
-	for (auto* image : m_imagesList) {
-		pen = image->pointPen();
-		pen.setColor(color);
-		image->setPointPen(pen);
-	}
-
-	m_initializing = true;
-	GuiTools::updatePenStyles(ui.cbSymbolBorderStyle, color);
-	m_initializing = false;
-}
-
-void DatapickerImageWidget::pointsBorderWidthChanged(double value) {
-	if (m_initializing)
-		return;
-
-	QPen pen;
-	for (auto* image : m_imagesList) {
-		pen = image->pointPen();
-		pen.setWidthF( Worksheet::convertToSceneUnits(value, Worksheet::Unit::Point) );
-		image->setPointPen(pen);
-	}
-}
-
 void DatapickerImageWidget::pointsVisibilityChanged(bool state) {
 	if (m_initializing)
 		return;
@@ -655,7 +443,7 @@ void DatapickerImageWidget::intensitySpanChanged(int lowerLimit, int upperLimit)
 	if (m_initializing)
 		return;
 
-	DatapickerImage::EditorSettings settings = m_image->settings();
+	auto settings = m_image->settings();
 	settings.intensityThresholdHigh = upperLimit;
 	settings.intensityThresholdLow = lowerLimit;
 	for (auto* image : m_imagesList)
@@ -666,7 +454,7 @@ void DatapickerImageWidget::foregroundSpanChanged(int lowerLimit, int upperLimit
 	if (m_initializing)
 		return;
 
-	DatapickerImage::EditorSettings settings = m_image->settings();
+	auto settings = m_image->settings();
 	settings.foregroundThresholdHigh = upperLimit;
 	settings.foregroundThresholdLow = lowerLimit;
 	for (auto* image : m_imagesList)
@@ -677,7 +465,7 @@ void DatapickerImageWidget::hueSpanChanged(int lowerLimit, int upperLimit) {
 	if (m_initializing)
 		return;
 
-	DatapickerImage::EditorSettings settings = m_image->settings();
+	auto settings = m_image->settings();
 	settings.hueThresholdHigh = upperLimit;
 	settings.hueThresholdLow = lowerLimit;
 	for (auto* image : m_imagesList)
@@ -688,7 +476,7 @@ void DatapickerImageWidget::saturationSpanChanged(int lowerLimit, int upperLimit
 	if (m_initializing)
 		return;
 
-	DatapickerImage::EditorSettings settings = m_image->settings();
+	auto settings = m_image->settings();
 	settings.saturationThresholdHigh = upperLimit;
 	settings.saturationThresholdLow = lowerLimit;
 	for (auto* image : m_imagesList)
@@ -699,7 +487,7 @@ void DatapickerImageWidget::valueSpanChanged(int lowerLimit, int upperLimit) {
 	if (m_initializing)
 		return;
 
-	DatapickerImage::EditorSettings settings = m_image->settings();
+	auto settings = m_image->settings();
 	settings.valueThresholdHigh = upperLimit;
 	settings.valueThresholdLow = lowerLimit;
 	for (auto* image : m_imagesList)
@@ -741,22 +529,6 @@ void DatapickerImageWidget::pointSeparationChanged(int value) {
 //*******************************************************************
 //******** SLOTs for changes triggered in DatapickerImage ***********
 //*******************************************************************
-/*!
- * called when the name or comment of image's parent (datapicker) was changed.
- */
-void DatapickerImageWidget::imageDescriptionChanged(const AbstractAspect* aspect) {
-	if (m_image->parentAspect() != aspect)
-		return;
-
-	m_initializing = true;
-	if (aspect->name() != ui.leName->text()) {
-		ui.leName->setText(aspect->name());
-	} else if (aspect->comment() != ui.leComment->text()) {
-		ui.leComment->setText(aspect->comment());
-	}
-	m_initializing = false;
-}
-
 void DatapickerImageWidget::imageFileNameChanged(const QString& name) {
 	m_initializing = true;
 	ui.leFileName->setText(name);
@@ -816,47 +588,6 @@ void DatapickerImageWidget::updateSymbolWidgets() {
 		ui.tSymbol->setEnabled(false);
 }
 
-void DatapickerImageWidget::symbolStyleChanged(Symbol::Style style) {
-	m_initializing = true;
-	ui.cbSymbolStyle->setCurrentIndex((int)style - 1);
-	m_initializing = false;
-}
-
-void DatapickerImageWidget::symbolSizeChanged(qreal size) {
-	m_initializing = true;
-	ui.sbSymbolSize->setValue( Worksheet::convertFromSceneUnits(size, Worksheet::Unit::Point) );
-	m_initializing = false;
-}
-
-void DatapickerImageWidget::symbolRotationAngleChanged(qreal angle) {
-	m_initializing = true;
-	ui.sbSymbolRotation->setValue(round(angle));
-	m_initializing = false;
-}
-
-void DatapickerImageWidget::symbolOpacityChanged(qreal opacity) {
-	m_initializing = true;
-	ui.sbSymbolOpacity->setValue( round(opacity*100.0) );
-	m_initializing = false;
-}
-
-void DatapickerImageWidget::symbolBrushChanged(const QBrush& brush) {
-	m_initializing = true;
-	ui.cbSymbolFillingStyle->setCurrentIndex((int) brush.style());
-	ui.kcbSymbolFillingColor->setColor(brush.color());
-	GuiTools::updateBrushStyles(ui.cbSymbolFillingStyle, brush.color());
-	m_initializing = false;
-}
-
-void DatapickerImageWidget::symbolPenChanged(const QPen& pen) {
-	m_initializing = true;
-	ui.cbSymbolBorderStyle->setCurrentIndex( (int) pen.style());
-	ui.kcbSymbolBorderColor->setColor( pen.color());
-	GuiTools::updatePenStyles(ui.cbSymbolBorderStyle, pen.color());
-	ui.sbSymbolBorderWidth->setValue( Worksheet::convertFromSceneUnits(pen.widthF(), Worksheet::Unit::Point));
-	m_initializing = false;
-}
-
 void DatapickerImageWidget::symbolVisibleChanged(bool on) {
 	m_initializing = true;
 	ui.chbSymbolVisible->setChecked(on);
@@ -874,10 +605,9 @@ void DatapickerImageWidget::load() {
 	ui.leFileName->setText( m_image->fileName() );
 
 	//highlight the text field for the background image red if an image is used and cannot be found
-	if (!m_image->fileName().isEmpty() && !QFile::exists(m_image->fileName()))
-		ui.leFileName->setStyleSheet("QLineEdit{background:red;}");
-	else
-		ui.leFileName->setStyleSheet(QString());
+	const QString& fileName = m_image->fileName();
+	bool invalid = (!fileName.isEmpty() && !QFile::exists(fileName));
+	GuiTools::highlight(ui.leFileName, invalid);
 
 	ui.cbGraphType->setCurrentIndex((int) m_image->axisPoints().type);
 	ui.sbTernaryScale->setValue(m_image->axisPoints().ternaryScale);
@@ -903,15 +633,6 @@ void DatapickerImageWidget::load() {
 	gvValue->setSpan(m_image->settings().valueThresholdLow, m_image->settings().valueThresholdHigh);
 	ui.sbPointSeparation->setValue(m_image->pointSeparation());
 	ui.sbMinSegmentLength->setValue(m_image->minSegmentLength());
-	ui.cbSymbolStyle->setCurrentIndex( (int)m_image->pointStyle() - 1 );
-	ui.sbSymbolSize->setValue( Worksheet::convertFromSceneUnits(m_image->pointSize(), Worksheet::Unit::Point) );
-	ui.sbSymbolRotation->setValue( m_image->pointRotationAngle() );
-	ui.sbSymbolOpacity->setValue( round(m_image->pointOpacity()*100.0) );
-	ui.cbSymbolFillingStyle->setCurrentIndex( (int) m_image->pointBrush().style() );
-	ui.kcbSymbolFillingColor->setColor(  m_image->pointBrush().color() );
-	ui.cbSymbolBorderStyle->setCurrentIndex( (int) m_image->pointPen().style() );
-	ui.kcbSymbolBorderColor->setColor( m_image->pointPen().color() );
-	ui.sbSymbolBorderWidth->setValue( Worksheet::convertFromSceneUnits(m_image->pointPen().widthF(), Worksheet::Unit::Point) );
 	ui.chbSymbolVisible->setChecked( m_image->pointVisibility() );
 	m_initializing = false;
 }
