@@ -1,38 +1,18 @@
-/***************************************************************************
+/*
     File                 : AsciiFilterTest.cpp
     Project              : LabPlot
     Description          : Tests for the ascii filter
     --------------------------------------------------------------------
-    Copyright            : (C) 2017 Alexander Semke (alexander.semke@web.de)
- ***************************************************************************/
+    SPDX-FileCopyrightText: 2017 Alexander Semke <alexander.semke@web.de>
 
-/***************************************************************************
- *                                                                         *
- *  This program is free software; you can redistribute it and/or modify   *
- *  it under the terms of the GNU General Public License as published by   *
- *  the Free Software Foundation; either version 2 of the License, or      *
- *  (at your option) any later version.                                    *
- *                                                                         *
- *  This program is distributed in the hope that it will be useful,        *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
- *  GNU General Public License for more details.                           *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the Free Software           *
- *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
- *   Boston, MA  02110-1301  USA                                           *
- *                                                                         *
- ***************************************************************************/
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "AsciiFilterTest.h"
 #include "backend/datasources/filters/AsciiFilter.h"
 #include "backend/spreadsheet/Spreadsheet.h"
 
 void AsciiFilterTest::initTestCase() {
-	const QString currentDir = __FILE__;
-	m_dataDir = currentDir.left(currentDir.lastIndexOf(QDir::separator())) + QDir::separator() + QLatin1String("data") + QDir::separator();
-
 	// needed in order to have the signals triggered by SignallingUndoCommand, see LabPlot.cpp
 	//TODO: redesign/remove this
 	qRegisterMetaType<const AbstractAspect*>("const AbstractAspect*");
@@ -48,7 +28,7 @@ void AsciiFilterTest::testEmptyFileAppend() {
 
 	const int rowCount = spreadsheet.rowCount();
 	const int colCount = spreadsheet.columnCount();
-	const QString fileName = m_dataDir + "empty_file.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/empty_file.txt"));
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Append;
 
 	filter.readDataFromFile(fileName, &spreadsheet, mode);
@@ -63,7 +43,7 @@ void AsciiFilterTest::testEmptyFilePrepend() {
 
 	const int rowCount = spreadsheet.rowCount();
 	const int colCount = spreadsheet.columnCount();
-	const QString fileName = m_dataDir + "empty_file.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/empty_file.txt"));
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Prepend;
 
 	filter.readDataFromFile(fileName, &spreadsheet, mode);
@@ -78,7 +58,7 @@ void AsciiFilterTest::testEmptyFileReplace() {
 
 	const int rowCount = spreadsheet.rowCount();
 	const int colCount = spreadsheet.columnCount();
-	const QString fileName = m_dataDir + "empty_file.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/empty_file.txt"));
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.readDataFromFile(fileName, &spreadsheet, mode);
 
@@ -89,7 +69,7 @@ void AsciiFilterTest::testEmptyFileReplace() {
 void AsciiFilterTest::testEmptyLines01() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "empty_lines_01.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/empty_lines_01.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter("auto");
@@ -121,7 +101,7 @@ void AsciiFilterTest::testEmptyLines01() {
 void AsciiFilterTest::testSparseFile01() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "sparse_file_01.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/sparse_file_01.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(",");
@@ -156,7 +136,7 @@ void AsciiFilterTest::testSparseFile01() {
 void AsciiFilterTest::testSparseFile02() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "sparse_file_02.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/sparse_file_02.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(",");
@@ -174,8 +154,8 @@ void AsciiFilterTest::testSparseFile02() {
 	QCOMPARE(spreadsheet.column(2)->name(), QLatin1String("Col2"));
 
 	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Integer);
-	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Numeric);
-	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	QCOMPARE(spreadsheet.column(0)->integerAt(0), 1);
 	QCOMPARE(spreadsheet.column(0)->integerAt(1), 2);
@@ -193,7 +173,7 @@ void AsciiFilterTest::testSparseFile02() {
 void AsciiFilterTest::testSparseFile03() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "sparse_file_03.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/sparse_file_03.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(",");
@@ -211,8 +191,8 @@ void AsciiFilterTest::testSparseFile03() {
 	QCOMPARE(spreadsheet.column(2)->name(), QLatin1String("Col2"));
 
 	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Integer);
-	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Numeric);
-	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	QCOMPARE(spreadsheet.column(0)->integerAt(0), 1);
 	QCOMPARE(spreadsheet.column(0)->integerAt(1), 2);
@@ -236,7 +216,7 @@ void AsciiFilterTest::testSparseFile03() {
 void AsciiFilterTest::testHeader01() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "separator_semicolon.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/separator_semicolon.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(";");
@@ -251,7 +231,7 @@ void AsciiFilterTest::testHeader01() {
 void AsciiFilterTest::testHeader02() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "separator_semicolon.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/separator_semicolon.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(";");
@@ -271,7 +251,7 @@ void AsciiFilterTest::testHeader02() {
 void AsciiFilterTest::testHeader03() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "separator_semicolon.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/separator_semicolon.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(";");
@@ -287,7 +267,7 @@ void AsciiFilterTest::testHeader03() {
 void AsciiFilterTest::testHeader04() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "separator_semicolon.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/separator_semicolon.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(";");
@@ -303,7 +283,7 @@ void AsciiFilterTest::testHeader04() {
 void AsciiFilterTest::testHeader05() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "separator_semicolon.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/separator_semicolon.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(";");
@@ -320,7 +300,7 @@ void AsciiFilterTest::testHeader05() {
 void AsciiFilterTest::testHeader06() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "separator_semicolon.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/separator_semicolon.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(";");
@@ -342,12 +322,13 @@ void AsciiFilterTest::testHeader06() {
 void AsciiFilterTest::testHeader07() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "comment_header_comment.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/comment_header_comment.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter("TAB");
-	filter.setStartRow(2);
+	filter.setHeaderLine(2);
 	filter.setHeaderEnabled(true);
+	filter.setDateTimeFormat(QLatin1String("yyyy-MM-dd hh:mm:ss.zzz"));
 	filter.readDataFromFile(fileName, &spreadsheet, mode);
 
 	//spreadsheet size
@@ -361,8 +342,8 @@ void AsciiFilterTest::testHeader07() {
 
 	//data types
 	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Integer);
-	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Numeric);
-	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	//values
 	QCOMPARE(spreadsheet.column(0)->integerAt(0), 0);
@@ -382,6 +363,51 @@ void AsciiFilterTest::testHeader07() {
 	QCOMPARE(spreadsheet.column(2)->valueAt(3), 0.3433);
 }
 
+/*!
+ * test with a file containing the header in the second line
+ * with a subsequent comment line ignored by using startRow.
+ */
+void AsciiFilterTest::testHeader07a() {
+	Spreadsheet spreadsheet("test", false);
+	AsciiFilter filter;
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/comment_header_comment.txt"));
+
+	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
+	filter.setSeparatingCharacter("TAB");
+	filter.setHeaderLine(2);
+	filter.setHeaderEnabled(true);
+	filter.setStartRow(4);
+	filter.setDateTimeFormat(QLatin1String("yyyy-MM-dd hh:mm:ss.zzz"));
+	filter.readDataFromFile(fileName, &spreadsheet, mode);
+
+	//spreadsheet size
+	QCOMPARE(spreadsheet.columnCount(), 3);
+	QCOMPARE(spreadsheet.rowCount(), 3);
+
+	//column names
+	QCOMPARE(spreadsheet.column(0)->name(), QLatin1String("counter"));
+	QCOMPARE(spreadsheet.column(1)->name(), QLatin1String("t[min]"));
+	QCOMPARE(spreadsheet.column(2)->name(), QLatin1String("#1ch1"));
+
+	//data types
+	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Integer);
+	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Double);
+
+	//values
+	QCOMPARE(spreadsheet.column(0)->integerAt(0), 1);
+	QCOMPARE(spreadsheet.column(1)->valueAt(0), 0.0513);
+	QCOMPARE(spreadsheet.column(2)->valueAt(0), 0.3448);
+
+	QCOMPARE(spreadsheet.column(0)->integerAt(1), 2);
+	QCOMPARE(spreadsheet.column(1)->valueAt(1), 0.1005);
+	QCOMPARE(spreadsheet.column(2)->valueAt(1), 0.3418);
+
+	QCOMPARE(spreadsheet.column(0)->integerAt(2), 3);
+	QCOMPARE(spreadsheet.column(1)->valueAt(2), 0.1516);
+	QCOMPARE(spreadsheet.column(2)->valueAt(2), 0.3433);
+}
+
 /*!
  * the header contains spaces in the column names, values are tab separated.
  * when using "auto" for the separator characters, the tab character has to
@@ -390,7 +416,7 @@ void AsciiFilterTest::testHeader07() {
 void AsciiFilterTest::testHeader08() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "separator_tab_with_header_with_spaces.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/separator_tab_with_header_with_spaces.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter("auto");
@@ -423,7 +449,7 @@ void AsciiFilterTest::testHeader08() {
 void AsciiFilterTest::testColumnRange00() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "numeric_data.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/numeric_data.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter("auto");
@@ -434,9 +460,9 @@ void AsciiFilterTest::testColumnRange00() {
 	QCOMPARE(spreadsheet.rowCount(), 5);
 	QCOMPARE(spreadsheet.columnCount(), 3);
 
-	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Numeric);
-	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Numeric);
-	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	//check the values for the first line
 	QCOMPARE(spreadsheet.column(0)->valueAt(0), 1.716299);
@@ -447,7 +473,7 @@ void AsciiFilterTest::testColumnRange00() {
 void AsciiFilterTest::testColumnRange01() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "numeric_data.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/numeric_data.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter("auto");
@@ -460,9 +486,9 @@ void AsciiFilterTest::testColumnRange01() {
 	QCOMPARE(spreadsheet.columnCount(), 4);
 
 	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Integer);
-	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Numeric);
-	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Numeric);
-	QCOMPARE(spreadsheet.column(3)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(3)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	//check the values for the first line
 	QCOMPARE(spreadsheet.column(0)->integerAt(0), 1);
@@ -474,7 +500,7 @@ void AsciiFilterTest::testColumnRange01() {
 void AsciiFilterTest::testColumnRange02() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "numeric_data.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/numeric_data.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter("auto");
@@ -487,8 +513,8 @@ void AsciiFilterTest::testColumnRange02() {
 	QCOMPARE(spreadsheet.rowCount(), 5);
 	QCOMPARE(spreadsheet.columnCount(), 2);
 
-	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Numeric);
-	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	//check the values for the first line
 	QCOMPARE(spreadsheet.column(0)->valueAt(0), -0.485527);
@@ -498,7 +524,7 @@ void AsciiFilterTest::testColumnRange02() {
 void AsciiFilterTest::testColumnRange03() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "numeric_data.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/numeric_data.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter("auto");
@@ -513,8 +539,8 @@ void AsciiFilterTest::testColumnRange03() {
 	QCOMPARE(spreadsheet.columnCount(), 3);
 
 	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Integer);
-	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Numeric);
-	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	//check the values for the first line
 	QCOMPARE(spreadsheet.column(0)->integerAt(0), 1);
@@ -525,7 +551,7 @@ void AsciiFilterTest::testColumnRange03() {
 void AsciiFilterTest::testColumnRange04() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "numeric_data.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/numeric_data.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter("auto");
@@ -538,7 +564,7 @@ void AsciiFilterTest::testColumnRange04() {
 	QCOMPARE(spreadsheet.rowCount(), 5);
 	QCOMPARE(spreadsheet.columnCount(), 1);
 
-	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	//check the values for the first line
 	QCOMPARE(spreadsheet.column(0)->valueAt(0), -0.288690);
@@ -547,7 +573,7 @@ void AsciiFilterTest::testColumnRange04() {
 void AsciiFilterTest::testColumnRange05() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "numeric_data.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/numeric_data.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter("auto");
@@ -565,7 +591,7 @@ void AsciiFilterTest::testColumnRange05() {
 void AsciiFilterTest::testColumnRange06() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "numeric_data.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/numeric_data.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter("auto");
@@ -586,7 +612,7 @@ void AsciiFilterTest::testColumnRange06() {
 void AsciiFilterTest::testRowRange00() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "numeric_data.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/numeric_data.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter("auto");
@@ -599,9 +625,9 @@ void AsciiFilterTest::testRowRange00() {
 	QCOMPARE(spreadsheet.rowCount(), 3);
 	QCOMPARE(spreadsheet.columnCount(), 3);
 
-	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Numeric);
-	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Numeric);
-	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	//check the values for the first and for the last lines
 	QCOMPARE(spreadsheet.column(0)->valueAt(0), 1.711721);
@@ -616,7 +642,7 @@ void AsciiFilterTest::testRowRange00() {
 void AsciiFilterTest::testRowRange01() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "numeric_data.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/numeric_data.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter("auto");
@@ -629,9 +655,9 @@ void AsciiFilterTest::testRowRange01() {
 	QCOMPARE(spreadsheet.rowCount(), 3);
 	QCOMPARE(spreadsheet.columnCount(), 3);
 
-	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Numeric);
-	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Numeric);
-	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	//check the values for the first and for the last lines
 	QCOMPARE(spreadsheet.column(0)->valueAt(0), 1.711721);
@@ -646,7 +672,7 @@ void AsciiFilterTest::testRowRange01() {
 void AsciiFilterTest::testRowRange02() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "numeric_data.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/numeric_data.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter("auto");
@@ -666,7 +692,7 @@ void AsciiFilterTest::testRowRange02() {
 void AsciiFilterTest::testRowColumnRange00() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "numeric_data.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/numeric_data.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter("auto");
@@ -681,8 +707,8 @@ void AsciiFilterTest::testRowColumnRange00() {
 	QCOMPARE(spreadsheet.rowCount(), 3);
 	QCOMPARE(spreadsheet.columnCount(), 2);
 
-	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Numeric);
-	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Double);
+	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	//check the values for the first and for the last lines
 	QCOMPARE(spreadsheet.column(0)->valueAt(0), -0.485527);
@@ -703,7 +729,7 @@ void AsciiFilterTest::testRowColumnRange00() {
 void AsciiFilterTest::testQuotedStrings00() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "quoted_strings.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/quoted_strings.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(",");
@@ -718,7 +744,7 @@ void AsciiFilterTest::testQuotedStrings00() {
 	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Text);
 	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Integer);
 	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Integer);
-	QCOMPARE(spreadsheet.column(3)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(3)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	QCOMPARE(spreadsheet.column(0)->textAt(0), QLatin1String("a"));
 	QCOMPARE(spreadsheet.column(1)->integerAt(0), 1000);
@@ -739,7 +765,7 @@ void AsciiFilterTest::testQuotedStrings00() {
 void AsciiFilterTest::testQuotedStrings01() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "quoted_strings_with_header.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/quoted_strings_with_header.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(",");
@@ -762,7 +788,7 @@ void AsciiFilterTest::testQuotedStrings01() {
 	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Text);
 	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Integer);
 	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Integer);
-	QCOMPARE(spreadsheet.column(3)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(3)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	//values
 	QCOMPARE(spreadsheet.column(0)->textAt(0), QLatin1String("a"));
@@ -784,7 +810,7 @@ void AsciiFilterTest::testQuotedStrings01() {
 void AsciiFilterTest::testQuotedStrings02() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "quoted_strings_one_line.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/quoted_strings_one_line.txt"));
 
 	QCOMPARE(QFile::exists(fileName), true);
 
@@ -801,7 +827,7 @@ void AsciiFilterTest::testQuotedStrings02() {
 	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Text);
 	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Integer);
 	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Integer);
-	QCOMPARE(spreadsheet.column(3)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(3)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	QCOMPARE(spreadsheet.column(0)->textAt(0), QLatin1String("a"));
 	QCOMPARE(spreadsheet.column(1)->integerAt(0), 1000);
@@ -812,7 +838,7 @@ void AsciiFilterTest::testQuotedStrings02() {
 void AsciiFilterTest::testQuotedStrings03() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "quoted_strings_one_line_with_header.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/quoted_strings_one_line_with_header.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(",");
@@ -835,7 +861,7 @@ void AsciiFilterTest::testQuotedStrings03() {
 	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Text);
 	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Integer);
 	QCOMPARE(spreadsheet.column(2)->columnMode(), AbstractColumn::ColumnMode::Integer);
-	QCOMPARE(spreadsheet.column(3)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(3)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	//values
 	QCOMPARE(spreadsheet.column(0)->textAt(0), QLatin1String("a"));
@@ -851,7 +877,7 @@ void AsciiFilterTest::testQuotedStrings03() {
 void AsciiFilterTest::testComments00() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "multi_line_comment.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/multi_line_comment.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(",");
@@ -861,7 +887,7 @@ void AsciiFilterTest::testComments00() {
 
 	//data types
 	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Integer);
-	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	//values
 	QCOMPARE(spreadsheet.column(0)->integerAt(0), 1);
@@ -874,7 +900,7 @@ void AsciiFilterTest::testComments00() {
 void AsciiFilterTest::testComments01() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "multi_line_comment_with_empty_lines.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/multi_line_comment_with_empty_lines.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(",");
@@ -884,7 +910,7 @@ void AsciiFilterTest::testComments01() {
 
 	//data types
 	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::Integer);
-	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	//values
 	QCOMPARE(spreadsheet.column(0)->integerAt(0), 1);
@@ -900,7 +926,7 @@ void AsciiFilterTest::testComments01() {
 void AsciiFilterTest::testComments02() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "separator_semicolon_with_header.txt";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/separator_semicolon_with_header.txt"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setCommentCharacter("");
@@ -943,7 +969,7 @@ void AsciiFilterTest::testComments02() {
 void AsciiFilterTest::testDateTime00() {
 	Spreadsheet spreadsheet("test", false);
 	AsciiFilter filter;
-	const QString fileName = m_dataDir + "datetime_01.csv";
+	const QString& fileName = QFINDTESTDATA(QLatin1String("data/datetime_01.csv"));
 
 	AbstractFileFilter::ImportMode mode = AbstractFileFilter::ImportMode::Replace;
 	filter.setSeparatingCharacter(",");
@@ -961,7 +987,7 @@ void AsciiFilterTest::testDateTime00() {
 
 	//data types
 	QCOMPARE(spreadsheet.column(0)->columnMode(), AbstractColumn::ColumnMode::DateTime);
-	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Numeric);
+	QCOMPARE(spreadsheet.column(1)->columnMode(), AbstractColumn::ColumnMode::Double);
 
 	//values
 	QDateTime value = QDateTime::fromString(QLatin1String("01/01/2019 00:00:00"), QLatin1String("dd/MM/yyyy hh:mm:ss"));
